@@ -1,9 +1,9 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -16,23 +16,24 @@ Sometimes, we need an object's permissions to be remapped to other permissions
 when the object is used in specual ways.  This is rather hard, since we
 need the object's ordinary permissions intact so we can manage it.
 
-$Id: PermissionMapping.py 73418 2007-03-21 11:48:43Z mj $
+$Id: PermissionMapping.py 40222 2005-11-18 15:46:28Z andreasjung $
 """
 
 from cgi import escape
 
 import ExtensionClass, Acquisition
 from Globals import InitializeClass
+from zope.interface import implements
 
+from interfaces import IPermissionMappingSupport
 from Owned import UnownableOwner
 from Permission import pname
-from requestmethod import postonly
 
 
 class RoleManager:
 
-    # XXX: No security declarations?
-    
+    implements(IPermissionMappingSupport)
+
     def manage_getPermissionMapping(self):
         """Return the permission mapping for the object
 
@@ -85,7 +86,6 @@ class RoleManager:
             return self.manage_access(
                 REQUEST,
                 manage_tabs_message='The permission mapping has been updated')
-    manage_setPermissionMapping = postonly(manage_setPermissionMapping)
 
     def _isBeingUsedAsAMethod(self, REQUEST =None, wannaBe=0):
         try:
