@@ -1,37 +1,42 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
-# FOR A PARTICULAR PURPOSE
+# FOR A PARTICULAR PURPOSE.
 #
 ##############################################################################
-"""Access control package"""
+"""Access control package.
 
-__version__='$Revision: 1.182 $'[11:-2]
+$Id: User.py 40222 2005-11-18 15:46:28Z andreasjung $
+"""
 
 import os
-
-import Globals, socket, SpecialUsers,re
-from Globals import DTMLFile, MessageDialog, Persistent, PersistentMapping
-from App.Management import Navigation, Tabs
-from Acquisition import Implicit
-from OFS.SimpleItem import Item
+import re
+import socket
 from base64 import decodestring
-from App.ImageFile import ImageFile
-from requestmethod import postonly
-from Role import RoleManager, DEFAULTMAXLISTUSERS
-from PermissionRole import _what_not_even_god_should_do, rolesForPermissionOn
-import AuthEncoding
-from AccessControl import getSecurityManager
+
+import Globals
+from Acquisition import Implicit
+from App.Management import Navigation, Tabs
+from Globals import DTMLFile, MessageDialog, Persistent, PersistentMapping
+from OFS.SimpleItem import Item
 from zExceptions import Unauthorized, BadRequest
-from AccessControl.SecurityManagement import newSecurityManager
-from AccessControl.SecurityManagement import noSecurityManager
-from AccessControl.ZopeSecurityPolicy import _noroles
+from zope.interface import implements
+
+import AuthEncoding
+import SpecialUsers
+from interfaces import IStandardUserFolder
+from PermissionRole import _what_not_even_god_should_do, rolesForPermissionOn
+from Role import RoleManager, DEFAULTMAXLISTUSERS
+from SecurityManagement import getSecurityManager
+from SecurityManagement import newSecurityManager
+from SecurityManagement import noSecurityManager
+from ZopeSecurityPolicy import _noroles
 
 
 _marker=[]
@@ -271,7 +276,8 @@ class BasicUser(Implicit):
 
     def __len__(self): return 1
     def __str__(self): return self.getUserName()
-    __repr__=__str__
+    def __repr__(self):
+        return '<%s %r>' % (self.__class__.__name__, self.getUserName())
 
 
 class SimpleUser(BasicUser):
@@ -451,7 +457,7 @@ class BasicUserFolder(Implicit, Persistent, Navigation, Tabs, RoleManager,
     isAUserFolder=1
     maxlistusers = DEFAULTMAXLISTUSERS
 
-    encrypt_passwords = 0
+    encrypt_passwords = 1
 
     manage_options=(
         (
@@ -528,33 +534,28 @@ class BasicUserFolder(Implicit, Persistent, Navigation, Tabs, RoleManager,
     # Authors of custom user folders don't need to do anything special to
     # support these - they will just call the appropriate '_' methods that
     # user folder subclasses already implement.
-    def userFolderAddUser(self, name, password, roles, domains,
-                          REQUEST=None, **kw):
+    def userFolderAddUser(self, name, password, roles, domains, **kw):
         """API method for creating a new user object. Note that not all
            user folder implementations support dynamic creation of user
            objects."""
         if hasattr(self, '_doAddUser'):
             return self._doAddUser(name, password, roles, domains, **kw)
         raise NotImplementedError
-    userFolderAddUser = postonly(userFolderAddUser)
 
-    def userFolderEditUser(self, name, password, roles, domains,
-                           REQUEST=None, **kw):
+    def userFolderEditUser(self, name, password, roles, domains, **kw):
         """API method for changing user object attributes. Note that not
            all user folder implementations support changing of user object
            attributes."""
         if hasattr(self, '_doChangeUser'):
             return self._doChangeUser(name, password, roles, domains, **kw)
         raise NotImplementedError
-    userFolderEditUser = postonly(userFolderEditUser)
 
-    def userFolderDelUsers(self, names, REQUEST=None):
+    def userFolderDelUsers(self, names):
         """API method for deleting one or more user objects. Note that not
            all user folder implementations support deletion of user objects."""
         if hasattr(self, '_doDelUsers'):
             return self._doDelUsers(names)
         raise NotImplementedError
-    userFolderDelUsers = postonly(userFolderDelUsers)
 
 
     # -----------------------------------
@@ -825,7 +826,6 @@ class BasicUserFolder(Implicit, Persistent, Navigation, Tabs, RoleManager,
             if REQUEST is not None:
                 return self.manage_userFolderProperties(
                     REQUEST, manage_tabs_message='Saved changes.')
-    manage_setUserFolderProperties = postonly(manage_setUserFolderProperties)
 
     def _isPasswordEncrypted(self, pw):
         return AuthEncoding.is_encrypted(pw)
@@ -846,6 +846,7 @@ class BasicUserFolder(Implicit, Persistent, Navigation, Tabs, RoleManager,
 
         return 1
 
+
     def _addUser(self,name,password,confirm,roles,domains,REQUEST=None):
         if not name:
             return MessageDialog(
@@ -880,7 +881,7 @@ class BasicUserFolder(Implicit, Persistent, Navigation, Tabs, RoleManager,
                    action ='manage_main')
         self._doAddUser(name, password, roles, domains)
         if REQUEST: return self._mainUser(self, REQUEST)
-    _addUser = postonly(_addUser)
+
 
     def _changeUser(self,name,password,confirm,roles,domains,REQUEST=None):
         if password == 'password' and confirm == 'pconfirm':
@@ -918,7 +919,6 @@ class BasicUserFolder(Implicit, Persistent, Navigation, Tabs, RoleManager,
                    action ='manage_main')
         self._doChangeUser(name, password, roles, domains)
         if REQUEST: return self._mainUser(self, REQUEST)
-    _changeUser = postonly(_changeUser)
 
     def _delUsers(self,names,REQUEST=None):
         if not names:
@@ -928,7 +928,6 @@ class BasicUserFolder(Implicit, Persistent, Navigation, Tabs, RoleManager,
                    action ='manage_main')
         self._doDelUsers(names)
         if REQUEST: return self._mainUser(self, REQUEST)
-    _delUsers = postonly(_delUsers)
 
     def manage_users(self,submit=None,REQUEST=None,RESPONSE=None):
         """This method handles operations on users for the web based forms
@@ -986,7 +985,7 @@ class BasicUserFolder(Implicit, Persistent, Navigation, Tabs, RoleManager,
 
     def _setId(self, id):
         if id != self.id:
-            raise Globals.MessageDialog(
+            raise MessageDialog(
                 title='Invalid Id',
                 message='Cannot change the id of a UserFolder',
                 action ='./manage_main',)
@@ -1009,7 +1008,9 @@ class BasicUserFolder(Implicit, Persistent, Navigation, Tabs, RoleManager,
         """ returns true if domain auth mode is set to true"""
         return getattr(self, '_domain_auth_mode', None)
 
+
 class UserFolder(BasicUserFolder):
+
     """Standard UserFolder object
 
     A UserFolder holds User objects which contain information
@@ -1017,6 +1018,8 @@ class UserFolder(BasicUserFolder):
     UserFolders function chiefly to control access by authenticating
     users and binding them to a collection of roles."""
 
+    implements(IStandardUserFolder)
+
     meta_type='User Folder'
     id       ='acl_users'
     title    ='User Folder'
@@ -1098,10 +1101,6 @@ class UserFolder(BasicUserFolder):
 Globals.default__class_init__(UserFolder)
 
 
-
-
-
-
 def manage_addUserFolder(self,dtself=None,REQUEST=None,**ignored):
     """ """
     f=UserFolder()
@@ -1130,7 +1129,6 @@ addr_match=re.compile(r'((\d{1,3}\.){1,3}\*)|((\d{1,3}\.){3}\d{1,3})').match
 host_match=re.compile(r'(([\_0-9a-zA-Z\-]*\.)*[0-9a-zA-Z\-]*)').match
 
 
-
 def domainSpecMatch(spec, request):
     host=''
     addr=''
