@@ -36,7 +36,7 @@
   USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
   DAMAGE.
 
-  $Id: cAccessControl.c 40459 2005-12-01 22:44:07Z tseaver $
+  $Id: cAccessControl.c 41698 2006-02-19 12:21:40Z andreasjung $
 
   If you have questions regarding this software,
   contact:
@@ -48,8 +48,8 @@
 
 */
 
-#include "ExtensionClass.h"
-#include "Acquisition.h"
+#include "ExtensionClass/ExtensionClass.h"
+#include "Acquisition/Acquisition.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -365,8 +365,6 @@ static PyObject *permissionName(PyObject *name);
 
 static PyObject *SecurityManager_validate(SecurityManager *self, 
                                           PyObject *args);
-static PyObject *SecurityManager_validateValue(SecurityManager *self,
-                                               PyObject *args);
 static PyObject *SecurityManager_DTMLValidate(SecurityManager *self,
                                               PyObject *args);
 static PyObject *SecurityManager_checkPermission(SecurityManager *self, 
@@ -470,11 +468,6 @@ static PyMethodDef SecurityManager_methods[] = {
 		METH_VARARGS,
 		""
 	},
-	{"validateValue",
-		(PyCFunction)SecurityManager_validateValue,
-		METH_VARARGS,
-		""
-	},
 	{"checkPermission",
 		(PyCFunction)SecurityManager_checkPermission,
 		METH_VARARGS,
@@ -1327,23 +1320,6 @@ SecurityManager_validate(SecurityManager *self, PyObject *args)
                        accessed, container, name, value, self->context, roles);
 }
 
-static PyObject *
-SecurityManager_validateValue(SecurityManager *self, PyObject *args)
-{
-  PyObject *value=Py_None, *roles=NULL;
-  
-  if (unpacktuple2(args, "validateValue", 1, &value, &roles) < 0) return NULL;
-          
-  CHECK_SECURITY_MANAGER_STATE(self, NULL);
-  GET_SECURITY_MANAGER_VALIDATE(self, NULL);
-  
-  if (roles==NULL) 
-    return callfunction5(self->validate, 
-                         Py_None, Py_None, Py_None, value, self->context);
-  return callfunction6(self->validate, 
-                       Py_None, Py_None, Py_None, value, self->context, roles);
-}
-
 static PyObject *
 SecurityManager_DTMLValidate(SecurityManager *self, PyObject *args)
 {
@@ -2263,7 +2239,7 @@ void initcAccessControl(void) {
 
 	module = Py_InitModule3("cAccessControl",
 		cAccessControl_methods,
-		"$Id: cAccessControl.c 40459 2005-12-01 22:44:07Z tseaver $\n");
+		"$Id: cAccessControl.c 41698 2006-02-19 12:21:40Z andreasjung $\n");
 
 	aq_init(); /* For Python <= 2.1.1, aq_init() should be after
                       Py_InitModule(). */
