@@ -4,7 +4,7 @@
   All Rights Reserved.
 
   This software is subject to the provisions of the Zope Public License,
-  Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+  Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
   THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
   WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
   WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -12,10 +12,10 @@
 
  ****************************************************************************/
 
-#include "ExtensionClass.h"
+#include "ExtensionClass/ExtensionClass.h"
 
 #define _IN_ACQUISITION_C
-#include "Acquisition.h"
+#include "Acquisition/Acquisition.h"
 
 static ACQUISITIONCAPI AcquisitionCAPI;
 
@@ -1059,6 +1059,7 @@ Wrapper_acquire_method(Wrapper *self, PyObject *args, PyObject *kw)
   PyObject *expl=0, *defalt=0;
   int explicit=1;
   int containment=0;
+  PyObject *result; /* DM 2005-08-25: argument "default" ignored */
 
   UNLESS (PyArg_ParseTupleAndKeywords(
 	     args, kw, "O|OOOOi", acquire_args+1,
@@ -1070,12 +1071,34 @@ Wrapper_acquire_method(Wrapper *self, PyObject *args, PyObject *kw)
 
   if (filter==Py_None) filter=0;
 
+  /* DM 2005-08-25: argument "default" ignored -- fix it! */
+# if 0
   return Wrapper_findattr(self,name,filter,extra,OBJECT(self),1,
 			  explicit || 
 			  self->ob_type==(PyTypeObject*)&Wrappertype,
 			  explicit, containment);
+# else
+  result = Wrapper_findattr(self,name,filter,extra,OBJECT(self),1,
+			  explicit || 
+			  self->ob_type==(PyTypeObject*)&Wrappertype,
+			  explicit, containment);
+  if (result == NULL && defalt != NULL) {
+    /* as "Python/bltinmodule.c:builtin_getattr" turn
+       only 'AttributeError' into a default value, such
+       that e.g. "ConflictError" and errors raised by the filter
+       are not mapped to the default value.
+    */
+    if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
+      PyErr_Clear();
+      Py_INCREF(defalt);
+      result = defalt;
+    }
+  }
+  return result;
+# endif
 }
 
+
 static PyObject *
 Wrapper_inContextOf(Wrapper *self, PyObject *args)
 {
@@ -1593,7 +1616,7 @@ init_Acquisition(void)
   /* Create the module and add the functions */
   m = Py_InitModule4("_Acquisition", methods,
 	   "Provide base classes for acquiring objects\n\n"
-	   "$Id: _Acquisition.c 30617 2005-06-03 07:42:38Z tseaver $\n",
+	   "$Id: _Acquisition.c 40222 2005-11-18 15:46:28Z andreasjung $\n",
 		     OBJECT(NULL),PYTHON_API_VERSION);
 
   d = PyModule_GetDict(m);
