@@ -1,9 +1,9 @@
 /*****************************************************************************
 
-  Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+  Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
   
   This software is subject to the provisions of the Zope Public License,
-  Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+  Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
   THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
   WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
   WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -12,10 +12,10 @@
  ****************************************************************************/
 static char cDocumentTemplate_module_documentation[] = 
 ""
-"\n$Id: cDocumentTemplate.c 24763 2004-05-17 05:59:28Z philikon $"
+"\n$Id: cDocumentTemplate.c 40222 2005-11-18 15:46:28Z andreasjung $"
 ;
 
-#include "ExtensionClass.h"
+#include "ExtensionClass/ExtensionClass.h"
 
 static PyObject *py_isDocTemp=0, *py_blocks=0, *py_=0, *join=0, *py_acquire;
 static PyObject *py___call__, *py___roles__, *py_AUTHENTICATED_USER;
@@ -420,10 +420,11 @@ static struct PyMethodDef MM_methods[] = {
   {"_pop",  (PyCFunction) MM_pop,  0,
    "_pop() -- Remove and return the last data source added"}, 
   {"getitem",  (PyCFunction) MM_get,  METH_VARARGS,
-   "getitem(key[,call]) -- Get a value\n\n"
-   "Normally, callable objects that can be called without arguments are\n"
-   "called during retrieval. This can be suppressed by providing a\n"
-   "second argument that is false.\n"
+   "getitem(key[,call]) -- Get a value from the MultiDict\n\n"
+   "If call is true, callable objects that can be called without arguments are\n"
+   "called during retrieval.\n"
+   "If call is false, the object will be returns without any attempt to call it.\n"
+   "If not specified, call is false by default.\n"
   }, 
   {"has_key",  (PyCFunction) MM_has_key,  METH_VARARGS,
    "has_key(key) -- Test whether the mapping has the given key"
