@@ -1,32 +1,41 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
-# FOR A PARTICULAR PURPOSE
+# FOR A PARTICULAR PURPOSE.
 #
 ##############################################################################
 """Find support
 
-$Id: FindSupport.py 25263 2004-06-04 18:58:49Z slinkp $
+$Id: FindSupport.py 40222 2005-11-18 15:46:28Z andreasjung $
 """
-import sys, os, time, Globals, ExtensionClass
-from DocumentTemplate.DT_Util import Eval
-from AccessControl.Permission import name_trans
-from Globals import DTMLFile
-from DocumentTemplate.DT_Util import InstanceDict, TemplateDict
-from DateTime import DateTime
+
 from string import translate
-from AccessControl.DTML import RestrictedDTML
+
+import Globals, ExtensionClass
 from AccessControl import ClassSecurityInfo
+from AccessControl.DTML import RestrictedDTML
+from AccessControl.Permission import name_trans
+from DateTime import DateTime
+from DocumentTemplate.DT_Util import Eval
+from DocumentTemplate.DT_Util import InstanceDict, TemplateDict
+from Globals import DTMLFile
+from zope.interface import implements
+
+from interfaces import IFindSupport
+
 
 class FindSupport(ExtensionClass.Base):
+
     """Find support for Zope Folders"""
 
+    implements(IFindSupport)
+
 #findframe is deprecated
     manage_findFrame=DTMLFile('dtml/findFrame', globals())
     manage_findForm=DTMLFile('dtml/findForm', globals(),
@@ -250,7 +259,7 @@ class FindSupport(ExtensionClass.Base):
 
         return result
 
-
+Globals.InitializeClass(FindSupport)
 
 
 class td(RestrictedDTML, TemplateDict):
@@ -304,8 +313,6 @@ def role_match(ob, permission, roles, lt=type([]), tt=type(())):
     return 1
 
 
-Globals.InitializeClass(FindSupport)
-
 # Helper functions
 
 def absattr(attr):
