@@ -1,21 +1,21 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
-# FOR A PARTICULAR PURPOSE
+# FOR A PARTICULAR PURPOSE.
 #
 ##############################################################################
+"""Property sheets
 
-"""Property sheets"""
-
-__version__='$Revision: 1.89.4.10 $'[11:-2]
-
+$Id: PropertySheets.py 40222 2005-11-18 15:46:28Z andreasjung $
+"""
 import time,  App.Management, Globals, sys
+from webdav.interfaces import IWriteLock
 from webdav.WriteLockInterface import WriteLockInterface
 from ZPublisher.Converters import type_converters
 from Globals import DTMLFile, MessageDialog
@@ -392,7 +392,6 @@ class PropertySheet(Traversable, Persistent, Implicit):
             else: result[code].append(prop)
             return
 
-
     del propstat
     del propdesc
 
@@ -475,6 +474,7 @@ class Virtual:
     def v_self(self):
         return self.aq_parent.aq_parent
 
+
 class DefaultProperties(Virtual, PropertySheet, View):
     """The default property set mimics the behavior of old-style Zope
        properties -- it stores its property values in the instance of
@@ -533,8 +533,7 @@ class DAVProperties(Virtual, PropertySheet, View):
 
     def dav__resourcetype(self):
         vself=self.v_self()
-        if (isDavCollection(vself) or
-            getattr(aq_base(vself), 'isAnObjectManager', None)):
+        if isDavCollection(vself):
             return '<n:collection/>'
         return ''
 
@@ -568,7 +567,8 @@ class DAVProperties(Virtual, PropertySheet, View):
     def dav__supportedlock(self):
         vself = self.v_self()
         out = '\n'
-        if WriteLockInterface.isImplementedBy(vself):
+        if IWriteLock.providedBy(vself) or \
+                WriteLockInterface.isImplementedBy(vself):
             out += ('  <n:lockentry>\n'
                     '  <d:lockscope><d:exclusive/></d:lockscope>\n'
                     '  <d:locktype><d:write/></d:locktype>\n'
@@ -579,10 +579,10 @@ class DAVProperties(Virtual, PropertySheet, View):
         security = getSecurityManager()
         user = security.getUser().getId()
 
-
         vself = self.v_self()
         out = '\n'
-        if WriteLockInterface.isImplementedBy(vself):
+        if IWriteLock.providedBy(vself) or \
+                WriteLockInterface.isImplementedBy(vself):
             locks = vself.wl_lockValues(killinvalids=1)
             for lock in locks:
 
@@ -596,7 +596,6 @@ class DAVProperties(Virtual, PropertySheet, View):
 
         return out
 
-
 Globals.default__class_init__(DAVProperties)
 
 
@@ -790,7 +789,6 @@ class FixedSchema(PropertySheet):
 Globals.default__class_init__(FixedSchema)
 
 
-
 class vps(Base):
     """Virtual Propertysheets
 
