@@ -1,9 +1,9 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -42,6 +42,8 @@ class RestrictedCompileMode(AbstractCompileMode):
     # See concrete subclasses below.
 
     def __init__(self, source, filename):
+        if source:
+            source = '\n'.join(source.splitlines())
         self.rm = RestrictionMutator()
         AbstractCompileMode.__init__(self, source, filename)
 
@@ -206,6 +208,8 @@ class RFunction(RModule):
 
     def __init__(self, p, body, name, filename, globals):
         self.params = p
+        if body:
+            body = '\n'.join(body.splitlines())
         self.body = body
         self.name = name
         self.globals = globals or []
