@@ -12,7 +12,7 @@
 ##############################################################################
 """Example functional doctest
 
-$Id: testFunctionalDocTest.py 38610 2005-09-25 11:36:45Z shh $
+$Id: testFunctionalDocTest.py 40959 2005-12-21 15:45:06Z shh $
 """
 
 import os, sys
@@ -35,7 +35,7 @@ def setUp(self):
     ... """)
     HTTP/1.1 200 OK
     Content-Length: 5
-    Content-Type: text/plain
+    Content-Type: text/plain; charset=...
     <BLANKLINE>
     index
     '''
