@@ -1,9 +1,9 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -17,11 +17,15 @@ import Method, Basic, Property, AccessControl.Role, re
 
 from ZPublisher.mapply import mapply
 from ExtensionClass import Base
+from Acquisition import aq_base
 from App.FactoryDispatcher import FactoryDispatcher
 from ComputedAttribute import ComputedAttribute
 from Products.PythonScripts.PythonScript import PythonScript
 from zExceptions import BadRequest, Redirect
 import webdav.Collection
+import ZClasses._pmc
+
+import transaction
 
 import marshal
 
@@ -91,7 +95,18 @@ from OFS.misc_ import p_
 p_.ZClass_Icon=Globals.ImageFile('class.gif', globals())
 
 class PersistentClass(Base, webdav.Collection.Collection ):
-    def __class_init__(self): pass
+
+    __metaclass__ = ZClasses._pmc.ZClassPersistentMetaClass
+
+    # We need this class to be treated as a normal global class, even
+    # though it is an instance of ZClassPersistentMetaClass.
+    # Subclasses should be stored in the database.  See
+    # _pmc._p_DataDescr.__get__.
+    
+    __global_persistent_class_not_stored_in_DB__ = True
+    
+    def __class_init__(self):
+        pass
 
 manage_addZClassForm=Globals.DTMLFile(
     'dtml/addZClass', globals(),
@@ -114,12 +129,6 @@ def find_class(ob, name):
             continue
         raise AttributeError, name
 
-def dbVersionEquals(ver):
-    # A helper function to isolate db version checking.
-    return hasattr(Globals, 'DatabaseVersion') and \
-       Globals.DatabaseVersion == ver
-
-
 bad_id=re.compile('[^a-zA-Z0-9_]').search
 
 def manage_addZClass(self, id, title='', baseclasses=[],
@@ -127,6 +136,7 @@ def manage_addZClass(self, id, title='', baseclasses=[],
                      zope_object=0):
     """Add a Z Class
     """
+    
     if bad_id(id) is not None:
         raise BadRequest, (
             'The id %s is invalid as a class name.' % id)
@@ -301,17 +311,14 @@ class ZClass( Base
                             hasattr(self._zclass_, '_p_deactivate')
                             )
 
-        copy._zclass_.__dict__.update(
-            self._zclass_.__dict__)
-        get_transaction().register(
-            copy._zclass_)
+        copy._zclass_.__dict__.update(self._zclass_.__dict__)
+        transaction.get().register(copy._zclass_)
         self._p_jar.exchange(self._zclass_, copy._zclass_)
         self._zclass_=copy._zclass_
 
         copy._zclass_propertysheets_class.__dict__.update(
             self._zclass_propertysheets_class.__dict__)
-        get_transaction().register(
-            copy._zclass_propertysheets_class)
+        transaction.get().register(copy._zclass_propertysheets_class)
         self._p_jar.exchange(self._zclass_propertysheets_class,
                              copy._zclass_propertysheets_class)
         self._zclass_propertysheets_class=copy._zclass_propertysheets_class
@@ -319,8 +326,7 @@ class ZClass( Base
         if hasattr(self.propertysheets.__class__, '_p_oid'):
             copy.propertysheets.__class__.__dict__.update(
                 self.propertysheets.__class__.__dict__)
-            get_transaction().register(
-                copy.propertysheets.__class__)
+            transaction.get().register(copy.propertysheets.__class__)
             self._p_jar.exchange(self.propertysheets.__class__,
                                  copy.propertysheets.__class__)
 
@@ -339,8 +345,6 @@ class ZClass( Base
 
     changeClassId__roles__ = ()  # Private
     def changeClassId(self, newid=None):
-        if not dbVersionEquals('3'):
-            return
         if newid is None: newid=self._new_class_id()
         self._unregister()
         if newid:
@@ -385,7 +389,7 @@ class ZClass( Base
             product=product,
             id=self.id,
             meta_type=z.meta_type or '',
-            meta_class=self,
+            meta_class=aq_base(self),
             )
 
     def _unregister(self):
@@ -415,16 +419,12 @@ class ZClass( Base
         self.propertysheets.methods.manage_afterClone(item)
 
     def manage_afterAdd(self, item, container):
-        if not dbVersionEquals('3'):
-            return
         if not self._zclass_.__module__:
             self.setClassAttr('__module__', self._new_class_id())
         self._register()
         self.propertysheets.methods.manage_afterAdd(item, container)
 
     def manage_beforeDelete(self, item, container):
-        if not dbVersionEquals('3'):
-            return
         self._unregister()
         self.propertysheets.methods.manage_beforeDelete(item, container)
 
@@ -449,10 +449,7 @@ class ZClass( Base
         screen of the new instance's parent Folder. Otherwise,
         the instance will be returned.
         """
-        i=mapply(self._zclass_, (), REQUEST)
-        try: i._setId(id)
-        except AttributeError:
-            i.id=id
+        i = self.fromRequest(id, REQUEST)
         folder=durl=None
         if hasattr(self, 'Destination'):
             d=self.Destination
@@ -462,6 +459,8 @@ class ZClass( Base
         if not hasattr(folder,'_setObject'):
             folder=folder.aq_parent
 
+        # An object is not guarenteed to have the id we passed in.
+        id = i.getId()
         folder._setObject(id, i)
 
         if RESPONSE is not None:
@@ -469,16 +468,23 @@ class ZClass( Base
             except: durl=REQUEST['URL3']
             RESPONSE.redirect(durl+'/manage_workspace')
         else:
-            # An object is not guarenteed to have the id we passed in.
-            id = i.getId()
             return folder._getOb(id)
 
     index_html=createInObjectManager
 
     def fromRequest(self, id=None, REQUEST={}):
-        i=mapply(self._zclass_, (), REQUEST)
-        if id is not None and (not hasattr(i, 'id') or not i.id): i.id=id
-
+        if self._zclass_.__init__ is object.__init__:
+            # there is no user defined __init__, avoid calling
+            # mapply then, as it would fail while trying
+            # to figure out the function properties
+            i = self._zclass_()
+        else:
+            i = mapply(self._zclass_, (), REQUEST)
+        if id is not None:
+            try:
+                i._setId(id)
+            except AttributeError:
+                i.id = id
         return i
 
     def __call__(self, *args, **kw):
@@ -519,16 +525,16 @@ class ZClass( Base
     def setClassAttr(self, name, value):
         c=self._zclass_
         setattr(c, name, value)
-        if not c._p_changed:
-            get_transaction().register(c)
+        if (not c._p_changed) and (c._p_jar is not None):
+            transaction.get().register(c)
             c._p_changed=1
 
     delClassAttr__roles__ = ()  # Private
     def delClassAttr(self, name):
         c=self._zclass_
         delattr(c, name)
-        if not c._p_changed:
-            get_transaction().register(c)
+        if (not c._p_changed) and (c._p_jar is not None):
+            transaction.get().register(c)
             c._p_changed=1
 
     def classDefinedPermissions(self):
