@@ -4,7 +4,7 @@
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -16,28 +16,29 @@ import os
 import sys
 import time
 import random
-import socket
 import asyncore
 import tempfile
 import threading
-
-import zLOG
+import logging
 
 import ZEO.ServerStub
 from ZEO.ClientStorage import ClientStorage
 from ZEO.Exceptions import ClientDisconnected
 from ZEO.zrpc.marshal import Marshaller
-from ZEO.zrpc.error import DisconnectedError
 from ZEO.tests import forker
 
 from ZODB.DB import DB
-from ZODB.Transaction import get_transaction, Transaction
 from ZODB.POSException import ReadOnlyError, ConflictError
 from ZODB.tests.StorageTestBase import StorageTestBase
 from ZODB.tests.MinPO import MinPO
 from ZODB.tests.StorageTestBase \
      import zodb_pickle, zodb_unpickle, handle_all_serials, handle_serials
 
+import transaction
+from transaction import Transaction
+
+logger = logging.getLogger('ZEO.tests.ConnectionTests')
+
 ZERO = '\0'*8
 
 class TestServerStub(ZEO.ServerStub.StorageServer):
@@ -91,7 +92,7 @@ class CommonSetupTearDown(StorageTestBase):
         for i in 1, 2, ...
         """
         self.__super_setUp()
-        zLOG.LOG("testZEO", zLOG.INFO, "setUp() %s" % self.id())
+        logging.info("setUp() %s", self.id())
         self.file = tempfile.mktemp()
         self.addr = []
         self._pids = []
@@ -103,13 +104,13 @@ class CommonSetupTearDown(StorageTestBase):
 
     def tearDown(self):
         """Try to cause the tests to halt"""
-        zLOG.LOG("testZEO", zLOG.INFO, "tearDown() %s" % self.id())
+        logging.info("tearDown() %s" % self.id())
         for p in self.conf_paths:
             os.remove(p)
         if getattr(self, '_storage', None) is not None:
             self._storage.close()
             if hasattr(self._storage, 'cleanup'):
-                zLOG.LOG("testZEO", zLOG.DEBUG, "cleanup storage %s" %
+                logging.debug("cleanup storage %s" %
                          self._storage.__name__)
                 self._storage.cleanup()
         for adminaddr in self._servers:
@@ -121,25 +122,27 @@ class CommonSetupTearDown(StorageTestBase):
                 os.waitpid(pid, 0)
         for c in self.caches:
             for i in 0, 1:
-                path = "c1-%s-%d.zec" % (c, i)
-                # On Windows before 2.3, we don't have a way to wait for
-                # the spawned server(s) to close, and they inherited
-                # file descriptors for our open files.  So long as those
-                # processes are alive, we can't delete the files.  Try
-                # a few times then give up.
-                need_to_delete = 0
-                if os.path.exists(path):
-                    need_to_delete = 1
-                    for dummy in range(5):
-                        try:
-                            os.unlink(path)
-                        except:
-                            time.sleep(0.5)
-                        else:
-                            need_to_delete = 0
-                            break
-                if need_to_delete:
-                    os.unlink(path)  # sometimes this is just gonna fail
+                for ext in "", ".trace":
+                    base = "%s-%s.zec%s" % (c, "1", ext)
+                    path = os.path.join(tempfile.tempdir, base)
+                    # On Windows before 2.3, we don't have a way to wait for
+                    # the spawned server(s) to close, and they inherited
+                    # file descriptors for our open files.  So long as those
+                    # processes are alive, we can't delete the files.  Try
+                    # a few times then give up.
+                    need_to_delete = False
+                    if os.path.exists(path):
+                        need_to_delete = True
+                        for dummy in range(5):
+                            try:
+                                os.unlink(path)
+                            except:
+                                time.sleep(0.5)
+                            else:
+                                need_to_delete = False
+                                break
+                    if need_to_delete:
+                        os.unlink(path)  # sometimes this is just gonna fail
         self.__super_tearDown()
 
     def _newAddr(self):
@@ -163,6 +166,7 @@ class CommonSetupTearDown(StorageTestBase):
         self.caches.append(cache)
         storage = TestClientStorage(self.addr,
                                     client=cache,
+                                    var=tempfile.tempdir,
                                     cache_size=cache_size,
                                     wait=wait,
                                     min_disconnect_poll=0.1,
@@ -188,9 +192,8 @@ class CommonSetupTearDown(StorageTestBase):
 
     def startServer(self, create=1, index=0, read_only=0, ro_svr=0, keep=None):
         addr = self.addr[index]
-        zLOG.LOG("testZEO", zLOG.INFO,
-                 "startServer(create=%d, index=%d, read_only=%d) @ %s" %
-                 (create, index, read_only, addr))
+        logging.info("startServer(create=%d, index=%d, read_only=%d) @ %s" %
+                     (create, index, read_only, addr))
         path = "%s.%d" % (self.file, index)
         sconf = self.getConfig(path, create, read_only)
         zconf = self.getServerConfig(addr, ro_svr)
@@ -203,8 +206,8 @@ class CommonSetupTearDown(StorageTestBase):
         self._servers.append(adminaddr)
 
     def shutdownServer(self, index=0):
-        zLOG.LOG("testZEO", zLOG.INFO, "shutdownServer(index=%d) @ %s" %
-                 (index, self._servers[index]))
+        logging.info("shutdownServer(index=%d) @ %s" %
+                     (index, self._servers[index]))
         adminaddr = self._servers[index]
         if adminaddr is not None:
             forker.shutdown_zeo_server(adminaddr)
@@ -258,8 +261,7 @@ class ConnectionTests(CommonSetupTearDown):
         self._storage.close()
 
     def checkMultipleServers(self):
-        # XXX crude test at first -- just start two servers and do a
-        # commit at each one.
+        # Crude test-- just start two servers and do a commit at each one.
 
         self._newAddr()
         self._storage = self.openClientStorage('test', 100000)
@@ -304,6 +306,8 @@ class ConnectionTests(CommonSetupTearDown):
         # Stores should fail here
         self.assertRaises(ReadOnlyError, self._dostore)
         self._storage.close()
+        # Get rid of the 'test left new threads behind' warning
+        time.sleep(0.1)
 
     def checkReadOnlyFallbackWritable(self):
         # Open a fallback client to a read-write server; stores succeed
@@ -329,7 +333,7 @@ class ConnectionTests(CommonSetupTearDown):
         self.assertRaises(ReadOnlyError, self._dostore)
         self._storage.close()
 
-    # XXX Compare checkReconnectXXX() here to checkReconnection()
+    # TODO:  Compare checkReconnectXXX() here to checkReconnection()
     # further down.  Is the code here hopelessly naive, or is
     # checkReconnection() overwrought?
 
@@ -389,7 +393,7 @@ class ConnectionTests(CommonSetupTearDown):
         # objects that weren't in the _seriald, because the client was
         # interrupted waiting for tpc_vote() to return.  When the next
         # transaction committed, it tried to do something with the
-        # bogus _tbuf entries.  The exaplanation is wrong/incomplete,
+        # bogus _tbuf entries.  The explanation is wrong/incomplete,
         # because tpc_begin() should clear the _tbuf.
 
         # 2003-01-15T15:44:19 ERROR(200) ZODB A storage error occurred
@@ -421,45 +425,122 @@ class ConnectionTests(CommonSetupTearDown):
         self.assertEqual(revid1, revid2)
         self._storage.close()
 
-    def checkRollover(self):
-        # Check that the cache works when the files are swapped.
-
-        # In this case, only one object fits in a cache file.  When the
-        # cache files swap, the first object is effectively uncached.
-
-        self._storage = self.openClientStorage('test', 1000)
+    def checkDisconnectedCacheWorks(self):
+        # Check that the cache works when the client is disconnected.
+        self._storage = self.openClientStorage('test')
         oid1 = self._storage.new_oid()
         obj1 = MinPO("1" * 500)
         self._dostore(oid1, data=obj1)
         oid2 = self._storage.new_oid()
         obj2 = MinPO("2" * 500)
         self._dostore(oid2, data=obj2)
+        expected1 = self._storage.load(oid1, '')
+        expected2 = self._storage.load(oid2, '')
+
+        # Shut it all down, and try loading from the persistent cache file
+        # without a server present.
         self._storage.close()
         self.shutdownServer()
-        self._storage = self.openClientStorage('test', 1000, wait=0)
-        self._storage.load(oid1, '')
-        self._storage.load(oid2, '')
+        self._storage = self.openClientStorage('test', wait=False)
+        self.assertEqual(expected1, self._storage.load(oid1, ''))
+        self.assertEqual(expected2, self._storage.load(oid2, ''))
+        self._storage.close()
+
+    def checkDisconnectedCacheFails(self):
+        # Like checkDisconnectedCacheWorks above, except the cache
+        # file is so small that only one object can be remembered.
+        self._storage = self.openClientStorage('test', cache_size=900)
+        oid1 = self._storage.new_oid()
+        obj1 = MinPO("1" * 500)
+        self._dostore(oid1, data=obj1)
+        oid2 = self._storage.new_oid()
+        obj2 = MinPO("2" * 500)
+        # The cache file is so small that adding oid2 will evict oid1.
+        self._dostore(oid2, data=obj2)
+        expected2 = self._storage.load(oid2, '')
+
+        # Shut it all down, and try loading from the persistent cache file
+        # without a server present.
+        self._storage.close()
+        self.shutdownServer()
+        self._storage = self.openClientStorage('test', cache_size=900,
+                                               wait=False)
+        # oid2 should still be in cache.
+        self.assertEqual(expected2, self._storage.load(oid2, ''))
+        # But oid1 should have been purged, so that trying to load it will
+        # try to fetch it from the (non-existent) ZEO server.
+        self.assertRaises(ClientDisconnected, self._storage.load, oid1, '')
+        self._storage.close()
+
+    def checkVerificationInvalidationPersists(self):
+        # This tests a subtle invalidation bug from ZODB 3.3:
+        # invalidations processed as part of ZEO cache verification acted
+        # kinda OK wrt the in-memory cache structures, but had no effect
+        # on the cache file.  So opening the file cache again could
+        # incorrectly believe that a previously invalidated object was
+        # still current.  This takes some effort to set up.
+
+        # First, using a persistent cache ('test'), create an object
+        # MinPO(13).  We used to see this again at the end of this test,
+        # despite that we modify it, and despite that it gets invalidated
+        # in 'test', before the end.
+        self._storage = self.openClientStorage('test')
+        oid = self._storage.new_oid()
+        obj = MinPO(13)
+        self._dostore(oid, data=obj)
+        self._storage.close()
+
+        # Now modify obj via a temp connection.  `test` won't learn about
+        # this until we open a connection using `test` again.
+        self._storage = self.openClientStorage()
+        pickle, rev = self._storage.load(oid, '')
+        newobj = zodb_unpickle(pickle)
+        self.assertEqual(newobj, obj)
+        newobj.value = 42 # .value *should* be 42 forever after now, not 13
+        self._dostore(oid, data=newobj, revid=rev)
+        self._storage.close()
+
+        # Open 'test' again.  `oid` in this cache should be (and is)
+        # invalidated during cache verification.  The bug was that it
+        # got invalidated (kinda) in memory, but not in the cache file.
+        self._storage = self.openClientStorage('test')
+
+        # The invalidation happened already.  Now create and store a new
+        # object before closing this storage:  this is so `test` believes
+        # it's seen transactions beyond the one that invalidated `oid`, so
+        # that the *next* time we open `test` it doesn't process another
+        # invalidation for `oid`.  It's also important that we not try to
+        # load `oid` now:  because it's been (kinda) invalidated in the
+        # cache's memory structures, loading it now would fetch the
+        # current revision from the server, thus hiding the bug.
+        obj2 = MinPO(666)
+        oid2 = self._storage.new_oid()
+        self._dostore(oid2, data=obj2)
+        self._storage.close()
+
+        # Finally, open `test` again and load `oid`.  `test` believes
+        # it's beyond the transaction that modified `oid`, so its view
+        # of whether it has an up-to-date `oid` comes solely from the disk
+        # file, unaffected by cache verification.
+        self._storage = self.openClientStorage('test')
+        pickle, rev = self._storage.load(oid, '')
+        newobj_copy = zodb_unpickle(pickle)
+        # This used to fail, with
+        #     AssertionError: MinPO(13) != MinPO(42)
+        # That is, `test` retained a stale revision of the object on disk.
+        self.assertEqual(newobj_copy, newobj)
         self._storage.close()
 
     def checkReconnection(self):
         # Check that the client reconnects when a server restarts.
 
-        # XXX Seem to get occasional errors that look like this:
-        # File ZEO/zrpc2.py, line 217, in handle_request
-        # File ZEO/StorageServer.py, line 325, in storea
-        # File ZEO/StorageServer.py, line 209, in _check_tid
-        # StorageTransactionError: (None, <tid>)
-        # could system reconnect and continue old transaction?
-
         self._storage = self.openClientStorage()
         oid = self._storage.new_oid()
         obj = MinPO(12)
         self._dostore(oid, data=obj)
-        zLOG.LOG("checkReconnection", zLOG.INFO,
-                 "About to shutdown server")
+        logging.info("checkReconnection(): About to shutdown server")
         self.shutdownServer()
-        zLOG.LOG("checkReconnection", zLOG.INFO,
-                 "About to restart server")
+        logging.info("checkReconnection(): About to restart server")
         self.startServer(create=0)
         oid = self._storage.new_oid()
         obj = MinPO(12)
@@ -469,13 +550,12 @@ class ConnectionTests(CommonSetupTearDown):
                 break
             except ClientDisconnected:
                 # Maybe the exception mess is better now
-                zLOG.LOG("checkReconnection", zLOG.INFO,
-                         "Error after server restart; retrying.",
-                         error=sys.exc_info())
-                get_transaction().abort()
+                logging.info("checkReconnection(): Error after"
+                             " server restart; retrying.", exc_info=True)
+                transaction.abort()
             # Give the other thread a chance to run.
             time.sleep(0.1)
-        zLOG.LOG("checkReconnection", zLOG.INFO, "finished")
+        logging.info("checkReconnection(): finished")
         self._storage.close()
 
     def checkBadMessage1(self):
@@ -521,7 +601,7 @@ class ConnectionTests(CommonSetupTearDown):
     # transaction.  This is not really a connection test, but it needs
     # about the same infrastructure (several storage servers).
 
-    # XXX WARNING: with the current ZEO code, this occasionally fails.
+    # TODO: with the current ZEO code, this occasionally fails.
     # That's the point of this test. :-)
 
     def NOcheckMultiStorageTransaction(self):
@@ -559,7 +639,8 @@ class ConnectionTests(CommonSetupTearDown):
         r1 = c1.root()
 
         r1["a"] = MinPO("a")
-        get_transaction().commit()
+        transaction.commit()
+        self.assertEqual(r1._p_state, 0) # up-to-date
 
         db2 = DB(self.openClientStorage())
         r2 = db2.open().root()
@@ -567,22 +648,22 @@ class ConnectionTests(CommonSetupTearDown):
         self.assertEqual(r2["a"].value, "a")
 
         r2["b"] = MinPO("b")
-        get_transaction().commit()
+        transaction.commit()
 
-        # make sure the invalidation is received in the other client
-        for i in range(10):
-            c1._storage.sync()
-            if c1._invalidated.has_key(r1._p_oid):
+        # Make sure the invalidation is received in the other client.
+        # We've had problems with this timing out on "slow" and/or "very
+        # busy" machines, so we increase the sleep time on each trip, and
+        # are willing to wait quite a long time.
+        for i in range(20):
+            c1.sync()
+            if r1._p_state == -1:
                 break
-            time.sleep(0.1)
-        self.assert_(c1._invalidated.has_key(r1._p_oid))
+            time.sleep(i / 10.0)
+        self.assertEqual(r1._p_state, -1) # ghost
 
-        # force the invalidations to be applied...
-        c1.setLocalTransaction()
-        c1.getTransaction().register(c1)
-        c1.getTransaction().abort()
         r1.keys() # unghostify
         self.assertEqual(r1._p_serial, r2._p_serial)
+        self.assertEqual(r1["b"].value, "b")
 
         db2.close()
         db1.close()
@@ -609,6 +690,9 @@ class InvqTests(CommonSetupTearDown):
         revid2 = self._dostore(oid2)
         revid2 = self._dostore(oid2, revid2)
 
+        # sync() is needed to prevent invalidation for oid from arriving
+        # in the middle of the load() call.
+        perstorage.sync()
         perstorage.load(oid, '')
         perstorage.close()
 
@@ -836,7 +920,7 @@ class ReconnectionTests(CommonSetupTearDown):
         self.pollDown()
         self._storage.verify_result = None
         perstorage.verify_result = None
-        zLOG.LOG("testZEO", zLOG.INFO, '2ALLBEEF')
+        logging.info('2ALLBEEF')
         self.startServer(create=0, keep=0)
         self.pollUp()
         self.pollUp(storage=perstorage)
@@ -879,7 +963,7 @@ class TimeoutTests(CommonSetupTearDown):
         unless = self.failUnless
         self._storage = storage = self.openClientStorage()
         # Assert that the zeo cache is empty
-        unless(not storage._cache._index)
+        unless(not list(storage._cache.contents()))
         # Create the object
         oid = storage.new_oid()
         obj = MinPO(7)
@@ -897,7 +981,7 @@ class TimeoutTests(CommonSetupTearDown):
         # We expect finish to fail
         raises(ClientDisconnected, storage.tpc_finish, t)
         # The cache should still be empty
-        unless(not storage._cache._index)
+        unless(not list(storage._cache.contents()))
         # Load should fail since the object should not be in either the cache
         # or the server.
         raises(KeyError, storage.load, oid, '')
@@ -905,10 +989,10 @@ class TimeoutTests(CommonSetupTearDown):
     def checkTimeoutProvokingConflicts(self):
         eq = self.assertEqual
         raises = self.assertRaises
-        unless = self.failUnless
+        require = self.assert_
         self._storage = storage = self.openClientStorage()
-        # Assert that the zeo cache is empty
-        unless(not storage._cache._index)
+        # Assert that the zeo cache is empty.
+        require(not list(storage._cache.contents()))
         # Create the object
         oid = storage.new_oid()
         obj = MinPO(7)
@@ -920,20 +1004,32 @@ class TimeoutTests(CommonSetupTearDown):
         revid1b = storage.tpc_vote(t)
         revid1 = handle_serials(oid, revid1a, revid1b)
         storage.tpc_finish(t)
-        # Now do a store, sleeping before the finish so as to cause a timeout
+        # Now do a store, sleeping before the finish so as to cause a timeout.
         obj.value = 8
         t = Transaction()
         storage.tpc_begin(t)
         revid2a = storage.store(oid, revid1, zodb_pickle(obj), '', t)
         revid2b = storage.tpc_vote(t)
         revid2 = handle_serials(oid, revid2a, revid2b)
-        # Now sleep long enough for the storage to time out
-        time.sleep(3)
+
+        # Now sleep long enough for the storage to time out.
+        # This used to sleep for 3 seconds, and sometimes (but very rarely)
+        # failed then.  Now we try for a minute.  It typically succeeds
+        # on the second time thru the loop, and, since self.timeout is 1,
+        # it's typically faster now (2/1.8 ~= 1.11 seconds sleeping instead
+        # of 3).
+        deadline = time.time() + 60 # wait up to a minute
+        while time.time() < deadline:
+            if storage.is_connected():
+                time.sleep(self.timeout / 1.8)
+                storage.sync()
+            else:
+                break
         storage.sync()
-        unless(not storage.is_connected())
+        require(not storage.is_connected())
         storage._wait()
-        unless(storage.is_connected())
-        # We expect finish to fail
+        require(storage.is_connected())
+        # We expect finish to fail.
         raises(ClientDisconnected, storage.tpc_finish, t)
         # Now we think we've committed the second transaction, but we really
         # haven't.  A third one should produce a POSKeyError on the server,
@@ -943,20 +1039,20 @@ class TimeoutTests(CommonSetupTearDown):
         storage.tpc_begin(t)
         storage.store(oid, revid2, zodb_pickle(obj), '', t)
         raises(ConflictError, storage.tpc_vote, t)
-        # Even aborting won't help
+        # Even aborting won't help.
         storage.tpc_abort(t)
         storage.tpc_finish(t)
-        # Try again
+        # Try again.
         obj.value = 10
         t = Transaction()
         storage.tpc_begin(t)
         storage.store(oid, revid2, zodb_pickle(obj), '', t)
-        # Even aborting won't help
+        # Even aborting won't help.
         raises(ConflictError, storage.tpc_vote, t)
-        # Abort this one and try a transaction that should succeed
+        # Abort this one and try a transaction that should succeed.
         storage.tpc_abort(t)
         storage.tpc_finish(t)
-        # Now do a store, sleeping before the finish so as to cause a timeout
+        # Now do a store.
         obj.value = 11
         t = Transaction()
         storage.tpc_begin(t)
@@ -964,7 +1060,7 @@ class TimeoutTests(CommonSetupTearDown):
         revid2b = storage.tpc_vote(t)
         revid2 = handle_serials(oid, revid2a, revid2b)
         storage.tpc_finish(t)
-        # Now load the object and verify that it has a value of 11
+        # Now load the object and verify that it has a value of 11.
         data, revid = storage.load(oid, '')
         eq(zodb_unpickle(data), MinPO(11))
         eq(revid, revid2)
