@@ -1,9 +1,9 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -12,13 +12,15 @@
 ##############################################################################
 """ Basic ZPublisher request management.
 
-$Id: BaseRequest.py 41707 2006-02-19 18:53:08Z shh $
+$Id: BaseRequest.py 41705 2006-02-19 18:44:27Z shh $
 """
-
 from urllib import quote
 import xmlrpc
 from zExceptions import Forbidden
 
+from zope.event import notify
+from zope.app.publication.interfaces import EndRequestEvent
+
 UNSPECIFIED_ROLES=''
 
 try:
@@ -83,6 +85,7 @@ class BaseRequest:
     def close(self):
         self.other.clear()
         self._held=None
+        notify(EndRequestEvent(None, self))
 
     def processInputs(self):
         """Do any input processing that could raise errors
@@ -566,7 +569,6 @@ def old_validation(groups, request, auth,
 # Zope 3 by then :)
 
 import types
-import sys
 
 itypes = {}
 for name in ('NoneType', 'IntType', 'LongType', 'FloatType', 'StringType',
@@ -578,9 +580,8 @@ for name in ('NoneType', 'IntType', 'LongType', 'FloatType', 'StringType',
         itypes[getattr(types, name)] = 0
 
 # Python 2.4 no longer maintains the types module.
-if sys.version_info >= (2, 4):
-    itypes[set] = 0
-    itypes[frozenset] = 0
+itypes[set] = 0
+itypes[frozenset] = 0
 
 def typeCheck(obj, deny=itypes):
     # Return true if its ok to publish the type, false otherwise.
