@@ -1,9 +1,9 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -25,6 +25,9 @@ from TaintedString import TaintedString
 from maybe_lock import allocate_lock
 xmlrpc=None # Placeholder for module that we'll import if we have to.
 
+# This may get overwritten during configuration
+default_encoding = 'iso-8859-15'
+
 isCGI_NAME = {
         'SERVER_SOFTWARE' : 1,
         'SERVER_NAME' : 1,
@@ -522,7 +525,7 @@ class HTTPRequest(BaseRequest):
                                 if hasattr(converter,'convert_unicode'):
                                     item = converter.convert_unicode(item)
                                 else:
-                                    item = converter(item.encode('iso-8859-15'))
+                                    item = converter(item.encode(default_encoding))
                             else:
                                 item=converter(item)
 
@@ -1290,7 +1293,7 @@ class HTTPRequest(BaseRequest):
         return result+"</table>"
 
     def __repr__(self):
-        return "<%s, URL=%s>" % (self.__class__.__name__, self['URL'])
+        return "<%s, URL=%s>" % (self.__class__.__name__, self.get('URL'))
 
     def text(self):
         result="FORM\n\n"
@@ -1330,21 +1333,15 @@ class HTTPRequest(BaseRequest):
             if auth[:6].lower() == 'basic ':
                 if base64 is None: import base64
                 [name,password] = \
-                    base64.decodestring(auth.split()[-1]).split(':')
+                    base64.decodestring(auth.split()[-1]).split(':', 1)
                 return name, password
 
     def taintWrapper(self, enabled=TAINTING_ENABLED):
         return enabled and TaintRequestWrapper(self) or self
 
-    # Used by Five
-
-    def getPresentationSkin(self):
-        """see zope.component.interfaces.IPresentationRequest"""
-        return getattr(self, '_presentation_skin', None)
-
-    def setPresentationSkin(self, skin):
-        """see zope.publisher.interfaces.IPublicationRequest"""
-        self._presentation_skin = skin
+    def shiftNameToApplication(self):
+        """see zope.publisher.interfaces.http.IVirtualHostRequest"""
+        # this is needed for ++skin++
 
     def getURL(self):
         return self.URL
@@ -1422,7 +1419,8 @@ class FileUpload:
         if hasattr(file, '__methods__'): methods=file.__methods__
         else: methods= ['close', 'fileno', 'flush', 'isatty',
                         'read', 'readline', 'readlines', 'seek',
-                        'tell', 'truncate', 'write', 'writelines']
+                        'tell', 'truncate', 'write', 'writelines',
+                        '__iter__','next'] # see Collector 1837
 
         d=self.__dict__
         for m in methods:
@@ -1442,6 +1440,8 @@ class FileUpload:
         """
         return not not self.filename
 
+    def xreadlines(self):
+        return self
 
 parse_cookie_lock=allocate_lock()
 def parse_cookie(text,
@@ -1566,4 +1566,4 @@ def _filterPasswordFields(items):
 # if any trusted-proxies are defined in the configuration file.
 
 trusted_proxies = []
-    
+
