@@ -649,18 +649,6 @@ class ProcessInputsTests(unittest.TestCase):
         self.assertEquals(req.cookies['hmm'], '')
         self.assertEquals(req.cookies['baz'], 'gee')
 
-    def testResolveUrl(self):
-        # Check that ResolveUrl really raises the same error 
-        # it received from ZPublisher.BaseRequest.traverse
-        # collector entry 1944
-        from ZPublisher.HTTPRequest import HTTPRequest
-        from zExceptions import NotFound
-        env = TEST_ENVIRON.copy()
-        req = HTTPRequest(None, env, None)
-        req['PARENTS'] = ['Nobody', 'cares', 'here'] 
-        testmethod = req.resolve_url
-        self.assertRaises(NotFound, testmethod, 'http://localhost/does_not_exist')
-
 TEST_ENVIRON = {
     'CONTENT_TYPE': 'multipart/form-data; boundary=12345',
     'REQUEST_METHOD': 'POST',
@@ -696,6 +684,23 @@ class RequestTests( unittest.TestCase ):
         req.close()
         self.assertEqual(start_count, sys.getrefcount(s))  # The test
 
+    def testFileIterator(self):
+        # checks fileupload object supports the iterator protocol
+        # collector entry 1837
+        import sys
+        from StringIO import StringIO
+        s = StringIO(TEST_FILE_DATA)
+        env = TEST_ENVIRON.copy()
+        from ZPublisher.HTTPRequest import HTTPRequest
+        req = HTTPRequest(s, env, None)
+        req.processInputs()
+        f=req.form.get('file')
+        self.assertEqual(list(f),['test\n'])
+        f.seek(0)
+        self.assertEqual(f.next(),'test\n')
+        f.seek(0)
+        self.assertEqual(f.xreadlines(),f)
+	
 
 def test_suite():
     suite = unittest.TestSuite()
