@@ -1,48 +1,58 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
-# FOR A PARTICULAR PURPOSE
+# FOR A PARTICULAR PURPOSE.
 #
 ##############################################################################
-
 """WebDAV support - resource objects.
 
-$Id: Resource.py 30876 2005-06-21 15:33:34Z sidnei $
+$Id: Resource.py 69723 2006-08-22 10:05:31Z shh $
 """
 
-import sys
 import mimetypes
+import sys
 from urllib import unquote
 
-import Globals
 import ExtensionClass
-from Acquisition import aq_base
+import Globals
 from AccessControl import getSecurityManager
+from Acquisition import aq_base
+from zExceptions import BadRequest, MethodNotAllowed
+from zExceptions import Unauthorized, Forbidden, NotFound
+from zope.interface import implements
 from ZPublisher.HTTPRangeSupport import HTTPRangeInterface
-from zExceptions import Unauthorized, Forbidden
-from zExceptions import BadRequest, MethodNotAllowed 
 
-from WriteLockInterface import WriteLockInterface
+import davcmds
 import Lockable
 from common import absattr, urlfix, rfc1123_date, tokenFinder, urlbase
 from common import IfParser
 from common import isDavCollection
 from common import Locked, Conflict, PreconditionFailed
-import davcmds
+from interfaces import IDAVResource
+from interfaces import IWriteLock
+from WriteLockInterface import WriteLockInterface
+
+from zope.event import notify
+from OFS.event import ObjectClonedEvent
+import OFS.subscribers
+
 
 class Resource(ExtensionClass.Base, Lockable.LockableItem):
+
     """The Resource mixin class provides basic WebDAV support for
     non-collection objects. It provides default implementations
     for most supported WebDAV HTTP methods, however certain methods
     such as PUT should be overridden to ensure correct behavior in
     the context of the object type."""
 
+    implements(IDAVResource)
+
     __dav_resource__=1
 
     __http_methods__=('GET', 'HEAD', 'POST', 'PUT', 'DELETE', 'OPTIONS',
@@ -228,8 +238,9 @@ class Resource(ExtensionClass.Base, Lockable.LockableItem):
                 # We're locked, and no if header was passed in, so
                 # the client doesn't own a lock.
                 raise Locked, 'Resource is locked.'
-        elif WriteLockInterface.isImplementedBy(parent) and \
-             parent.wl_isLocked():
+        elif (IWriteLock.providedBy(parent) or
+                WriteLockInterface.isImplementedBy(parent)) and \
+                parent.wl_isLocked():
             if ifhdr:
                 parent.dav__simpleifhandler(REQUEST, RESPONSE, 'DELETE', col=1)
             else:
@@ -356,8 +367,9 @@ class Resource(ExtensionClass.Base, Lockable.LockableItem):
         if existing:
             # The destination itself exists, so we need to check its locks
             destob = aq_base(parent)._getOb(name)
-            if WriteLockInterface.isImplementedBy(destob) and \
-               destob.wl_isLocked():
+            if (IWriteLock.providedBy(destob) or
+                    WriteLockInterface.isImplementedBy(destob)) and \
+                    destob.wl_isLocked():
                 if ifhdr:
                     itrue = destob.dav__simpleifhandler(
                         REQUEST, RESPONSE, 'COPY', refresh=1)
@@ -365,8 +377,9 @@ class Resource(ExtensionClass.Base, Lockable.LockableItem):
                         raise PreconditonFailed
                 else:
                     raise Locked, 'Destination is locked.'
-        elif WriteLockInterface.isImplementedBy(parent) and \
-             parent.wl_isLocked():
+        elif (IWriteLock.providedBy(parent) or
+                WriteLockInterface.isImplementedBy(parent)) and \
+                parent.wl_isLocked():
             if ifhdr:
                 parent.dav__simpleifhandler(REQUEST, RESPONSE, 'COPY',
                                             refresh=1)
@@ -387,7 +400,11 @@ class Resource(ExtensionClass.Base, Lockable.LockableItem):
         parent._setObject(name, ob)
         ob = parent._getOb(name)
         ob._postCopy(parent, op=0)
-        ob.manage_afterClone(ob)
+
+        OFS.subscribers.compatibilityCall('manage_afterClone', ob, ob)
+
+        notify(ObjectClonedEvent(ob))
+
         # We remove any locks from the copied object because webdav clients
         # don't track the lock status and the lock token for copied resources
         ob.wl_clearLocks()
@@ -451,8 +468,9 @@ class Resource(ExtensionClass.Base, Lockable.LockableItem):
         if existing:
             # The destination itself exists, so we need to check its locks
             destob = aq_base(parent)._getOb(name)
-            if WriteLockInterface.isImplementedBy(destob) and \
-               destob.wl_isLocked():
+            if (IWriteLock.providedBy(destob) or
+                    WriteLockInterface.isImplementedBy(destob)) and \
+                    destob.wl_isLocked():
                 if ifhdr:
                     itrue = destob.dav__simpleifhandler(
                         REQUEST, RESPONSE, 'MOVE', url=dest, refresh=1)
@@ -460,8 +478,9 @@ class Resource(ExtensionClass.Base, Lockable.LockableItem):
                         raise PreconditionFailed
                 else:
                     raise Locked, 'Destination is locked.'
-        elif WriteLockInterface.isImplementedBy(parent) and \
-             parent.wl_isLocked():
+        elif (IWriteLock.providedBy(parent) or
+                WriteLockInterface.isImplementedBy(parent)) and \
+                parent.wl_isLocked():
             # There's no existing object in the destination folder, so
             # we need to check the folders locks since we're changing its
             # member list
@@ -598,5 +617,4 @@ class Resource(ExtensionClass.Base, Lockable.LockableItem):
     def listDAVObjects(self):
         return []
 
-        
 Globals.default__class_init__(Resource)
