@@ -13,25 +13,31 @@
 ##############################################################################
 """Server Control View
 
-$Id: servercontrol.py 27409 2004-09-02 07:05:38Z pruggera $
+$Id: servercontrol.py 40841 2005-12-17 11:30:12Z hdima $
 """
 __docformat__ = 'restructuredtext'
 
 from zope.app import zapi
 from zope.app.applicationcontrol.interfaces import IServerControl
 
-from zope.app.i18n import ZopeMessageIDFactory as _
+from zope.app.i18n import ZopeMessageFactory as _
+
 
 class ServerControlView(object):
 
     def serverControl(self):
         return zapi.getUtility(IServerControl)
 
-    def action(self, time=0):
+    def action(self):
         """Do the shutdown/restart!"""
+        control = self.serverControl()
+        time = self.request.get('time', 0)
+
         if 'restart' in self.request:
-            return (self.serverControl().restart(time)
-                    or _(u"You restarted the server."))
+            control.restart(time)
+            return _("The server will be restarted in ${number} seconds.",
+                mapping={"number": time})
         elif 'shutdown' in self.request:
-            return (self.serverControl().shutdown(time)
-                    or _("You shut down the server."))
+            control.shutdown(time)
+            return _("The server will be shutdown in ${number} seconds.",
+                mapping={"number": time})
