@@ -13,15 +13,26 @@
 ##############################################################################
 """Schema-generation tests
 
-$Id: tests.py 25177 2004-06-02 13:17:31Z jim $
+$Id: tests.py 29216 2005-02-19 00:08:56Z jim $
 """
 
 import unittest
-from zope.testing.doctestunit import DocTestSuite
+from zope.testing import doctest
+from zope.app.testing import placelesssetup
+
+
+def tearDownREADME(test):
+    placelesssetup.tearDown(test)
+    test.globs['db'].close()
 
 def test_suite():
     return unittest.TestSuite((
-        DocTestSuite('zope.app.generations.generations'),
+        doctest.DocFileSuite(
+            'README.txt',
+            setUp=placelesssetup.setUp, tearDown=tearDownREADME,
+            ),
+        doctest.DocTestSuite('zope.app.generations.generations'),
+        doctest.DocTestSuite('zope.app.generations.utility'),
         ))
 
 if __name__ == '__main__':
