@@ -13,7 +13,7 @@
 ##############################################################################
 """Browser Resource
 
-$Id: resource.py 26966 2004-08-09 18:55:24Z fdrake $
+$Id: resource.py 29143 2005-02-14 22:43:16Z srichter $
 """
 from zope.component.interfaces import IResource
 from zope.interface import implements
@@ -36,5 +36,5 @@ class Resource(Location):
             name = name[12:]
 
         site = getSite()
-        url = str(zapi.getViewProviding(site, IAbsoluteURL, self.request))
+        url = str(zapi.getMultiAdapter((site, self.request), IAbsoluteURL))
         return "%s/@@/%s" % (url, name)
