@@ -13,7 +13,7 @@
 ##############################################################################
 """Tests for the ZCML Documentation Module
 
-$Id: test_docstrings.py 25177 2004-06-02 13:17:31Z jim $
+$Id: test_docstrings.py 38178 2005-08-30 21:50:19Z mj $
 """
 import unittest
 from zope.testing.doctestunit import DocTestSuite
@@ -26,7 +26,7 @@ class LocaleInheritanceStub(AttributeInheritance):
 
     def getInheritedSelf(self):
         if self.__nextLocale__ is None:
-            raise NoParentException, 'No parent was specified.'
+            raise NoParentException('No parent was specified.')
         return self.__nextLocale__
 
 
