@@ -13,15 +13,16 @@
 ##############################################################################
 """Schema interfaces and exceptions
 
-$Id: interfaces.py 28315 2004-11-01 23:30:26Z fdrake $
+$Id: interfaces.py 39752 2005-10-30 20:16:09Z srichter $
 """
+__docformat__ = "reStructuredText"
 
 from zope.interface import Interface, Attribute
-from zope.schema._bootstrapfields import Field, Text, TextLine, Bool, Int
 from zope.schema._bootstrapfields import Container, Iterable
+from zope.schema._bootstrapfields import Field, Text, TextLine, Bool, Int
 
-from zope.i18nmessageid import MessageIDFactory
-_ = MessageIDFactory("zope")
+from zope.i18nmessageid import MessageFactory
+_ = MessageFactory("zope")
 
 # Import from _bootstrapinterfaces only because other packages will expect
 # to find these interfaces here.
@@ -136,7 +137,7 @@ class IField(Interface):
 
     order = Int(
         title=_(u"Field Order"),
-        description=_(u"""\
+        description=_(u"""
         The order attribute can be used to determine the order in
         which fields in a schema were defined. If one field is created
         after another (in the same thread), its order will be
@@ -184,23 +185,23 @@ class IIterable(IField):
     u"""Fields with a value that can be iterated over.
 
     The value needs to support iteration; the implementation mechanism
-    is not constrained.  (Either __iter__() or __getitem__() may be
+    is not constrained.  (Either `__iter__()` or `__getitem__()` may be
     used.)
     """
 
 class IContainer(IField):
-    u"""Fields whose value allows an 'x in value' check.
+    u"""Fields whose value allows an ``x in value`` check.
 
-    The Value needs to support the 'in' operator, but is not
-    constrained in how it does so (whether it defines __contains__()
-    or __getitem__() is immaterial).
+    The value needs to support the `in` operator, but is not
+    constrained in how it does so (whether it defines `__contains__()`
+    or `__getitem__()` is immaterial).
     """
 
 class IOrderable(IField):
     u"""Field requiring its value to be orderable.
 
     The set of value needs support a complete ordering; the
-    implementation mechanism is not constrained.  Either __cmp__() or
+    implementation mechanism is not constrained.  Either `__cmp__()` or
     'rich comparison' methods may be used.
     """
 
@@ -234,10 +235,10 @@ class IMinMaxLen(ILen):
 
     min_length = Int(
         title=_(u"Minimum length"),
-        description=_(u"""\
+        description=_(u"""
         Value after whitespace processing cannot have less than
-        min_length characters (if a string type) or elements (if
-        another sequence type). If min_length is None, there is
+        `min_length` characters (if a string type) or elements (if
+        another sequence type). If `min_length` is ``None``, there is
         no minimum.
         """),
         required=False,
@@ -246,11 +247,11 @@ class IMinMaxLen(ILen):
 
     max_length = Int(
         title=_(u"Maximum length"),
-        description=_(u"""\
+        description=_(u"""
         Value after whitespace processing cannot have greater
-        or equal than max_length characters (if a string type) or 
-        elements (if another sequence type). If max_length is
-        None, there is no maximum."""),
+        or equal than `max_length` characters (if a string type) or
+        elements (if another sequence type). If `max_length` is
+        ``None``, there is no maximum."""),
         required=False,
         min=0, # needs to be a positive number
         default=None)
@@ -284,6 +285,9 @@ class IASCII(IBytes):
 class IBytesLine(IBytes):
     u"""Field containing a byte string without newlines."""
 
+class IASCIILine(IASCII):
+    u"""Field containing a 7-bit ASCII string without newlines."""
+
 class IText(IMinMaxLen, IIterable, IField):
     u"""Field containing a unicode string."""
 
@@ -326,6 +330,9 @@ class IDatetime(IMinMax, IField):
 class IDate(IMinMax, IField):
     u"""Field containing a date."""
 
+class ITimedelta(IMinMax, IField):
+    u"""Field containing a timedelta."""
+
 def _is_field(value):
     if not IField.providedBy(value):
         return False
@@ -354,20 +361,12 @@ class IChoice(IField):
 
     Only one, values or vocabulary, may be specified for a given choice.
     """
-    vocabularyName = TextLine(
-        title=u"Vocabulary Name",
-        description=(u"The name of the vocabulary to be used.  This name\n"
-                     u"is intended to be used by the IVocabularyRegistry's\n"
-                     u"get() method."),
-        required=False,
-        default=None)
-
     vocabulary = Attribute(
         "vocabulary",
-        ("IBaseVocabulary to be used, or None.\n"
+        ("IBaseVocabulary to be used, or vocabulary name, or None.\n"
          "\n"
-         "If None, the vocabularyName should be used by an\n"
-         "IVocabularyRegistry should be used to locate an appropriate\n"
+         "If a string, the vocabulary name should be used by an\n"
+         "IVocabularyRegistry to locate an appropriate\n"
          "IBaseVocabulary object."))
 
 # Collections:
@@ -449,6 +448,7 @@ class ITokenizedTerm(ITerm):
     """Object representing a single value in a tokenized vocabulary.
     """
 
+    # TODO: There should be a more specialized field type for this.
     token = Attribute(
         "token",
         """Token which can be used to represent the value on a stream.
@@ -462,7 +462,51 @@ class ITitledTokenizedTerm(ITokenizedTerm):
     
     title = TextLine(title=_(u"Title"))
 
-class IBaseVocabulary(Interface):
+class ISource(Interface):
+    """A set of values from which to choose
+
+    Sources represent sets of values. They are used to specify the
+    source for choice fields.
+
+    Sources can be large (even infinite), in which case, they need to
+    be queried to find out what their values are.
+    
+    """
+
+    def __contains__(value):
+        """Return whether the value is available in this source
+        """
+
+class ISourceQueriables(Interface):
+    """A collection of objects for querying sources
+    """
+
+    def getQueriables():
+        """Return an iterable of objects that can be queried
+
+        The returned obects should be two-tuples with:
+
+        - A unicode id
+
+          The id must uniquely identify the queriable object within
+          the set of queryable objects. Furthermore, in subsequent
+          calls, the same id should be used for a given queriable
+          object.
+
+        - A queryable object
+
+          This is an object for which there is a view is provided for
+          searcing for items.
+
+        """
+
+class IContextSourceBinder(Interface):
+    def __call__(context):
+        """Return a context-bound instance that implements ISource.
+        """
+    
+
+class IBaseVocabulary(ISource):
     """Representation of a vocabulary.
 
     At this most basic level, a vocabulary only need to support a test
@@ -471,15 +515,27 @@ class IBaseVocabulary(Interface):
     vocabularies which are intrinsically ordered).
     """
 
-    def __contains__(value):
-        """Returns True if the value is available in this vocabulary."""
-
     def getTerm(value):
         """Return the ITerm object for the term 'value'.
 
         If 'value' is not a valid term, this method raises LookupError.
         """
 
+
+class IIterableSource(ISource):
+    """Source which supports iteration over allowed values.
+
+    The objects iteration provides must be values from the source.
+    """
+
+    def __iter__():
+        """Return an iterator which provides the values from the source."""
+
+    def __len__():
+        """Return the number of valid values, or sys.maxint."""
+
+
+# BBB vocabularies are pending deprecation, hopefully in 3.3
 class IIterableVocabulary(Interface):
     """Vocabulary which supports iteration over allowed values.
 
@@ -501,14 +557,16 @@ class IVocabulary(IIterableVocabulary, IBaseVocabulary):
 class IVocabularyTokenized(IVocabulary):
     """Vocabulary that provides support for tokenized representation.
 
-    This interface must be used as a mix-in with IBaseVocabulary.
-
     Terms returned from getTerm() and provided by iteration must
     conform to ITitledTokenizedTerm.
     """
 
     def getTermByToken(token):
-        """Return an ITokenizedTerm for the passed-in token."""
+        """Return an ITokenizedTerm for the passed-in token.
+
+        If `token` is not represented in the vocabulary, `LookupError`
+        is raised.
+        """
 
 
 class IVocabularyRegistry(Interface):
