@@ -23,12 +23,14 @@ messages about granted attribute access.
 Note that the ZOPE_WATCH_CHECKERS mechanism will eventually be
 replaced with a more general security auditing mechanism.
 
-$Id: checker.py 27188 2004-08-20 17:02:03Z jim $
+$Id: checker.py 40887 2005-12-19 18:03:34Z poster $
 """
 import os
 import sys
+import sets
 import types
 import datetime
+import pytz
 import weakref
 
 from zope.exceptions import DuplicationError
@@ -83,6 +85,57 @@ def ProxyFactory(object, checker=None):
 
 directlyProvides(ProxyFactory, ISecurityProxyFactory)
 
+def canWrite(obj, name):
+    """Check whether the interaction may write an attribute named name on obj.
+
+    Convenience method.  Rather than using checkPermission in high level code,
+    use canWrite and canAccess to avoid binding code to permissions.
+    """
+    obj = ProxyFactory(obj)
+    checker = getChecker(obj)
+    try:
+        checker.check_setattr(obj, name)
+    except Unauthorized:
+        return False
+    except ForbiddenAttribute:
+        # we are going to be a bit DWIM-y here: see
+        # http://www.zope.org/Collectors/Zope3-dev/506
+        
+        # generally, if the check is ForbiddenAttribute we want it to be
+        # raised: it probably indicates a programming or configuration error.
+        # However, we special case a write ForbiddenAttribute when one can
+        # actually read the attribute: this represents a reasonable
+        # configuration of a readonly attribute, and returning False (meaning
+        # "no, you can't write it") is arguably more useful than raising the
+        # exception.
+        try:
+            checker.check_getattr(obj, name)
+            # we'll let *this* ForbiddenAttribute fall through, if any.  It
+            # means that both read and write are forbidden.
+        except Unauthorized:
+            pass
+        return False
+    # all other exceptions, other than Unauthorized and ForbiddenAttribute,
+    # should be passed through uncaught, as they indicate programmer error
+    return True
+
+def canAccess(obj, name):
+    """Check whether the interaction may access an attribute named name on obj.
+
+    Convenience method.  Rather than using checkPermission in high level code,
+    use canWrite and canAccess to avoid binding code to permissions.
+    """
+    # access attributes and methods, including, in the current checker
+    # implementation, special names like __getitem__
+    obj = ProxyFactory(obj)
+    checker = getChecker(obj)
+    try:
+        checker.check_getattr(obj, name)
+    except Unauthorized:
+        return False
+    # if it is Forbidden (or anything else), let it be raised: it probably
+    # indicates a programming or configuration error
+    return True
 
 class Checker(object):
     implements(INameBasedChecker)
@@ -91,7 +144,7 @@ class Checker(object):
         """Create a checker
 
         A dictionary must be provided for computing permissions for
-        names. The disctionary get will be called with attribute names
+        names. The dictionary get will be called with attribute names
         and must return a permission id, None, or the special marker,
         CheckerPublic. If None is returned, then access to the name is
         forbidden. If CheckerPublic is returned, then access will be
@@ -101,7 +154,6 @@ class Checker(object):
         set attribute access.
 
         """
-
         assert isinstance(get_permissions, dict)
         self.get_permissions = get_permissions
         if set_permissions is not None:
@@ -135,10 +187,10 @@ class Checker(object):
                 return # allowed
             else:
                 __traceback_supplement__ = (TracebackSupplement, object)
-                raise Unauthorized(name, permission)
+                raise Unauthorized(object, name, permission)
 
         __traceback_supplement__ = (TracebackSupplement, object)
-        raise ForbiddenAttribute, (name, object)
+        raise ForbiddenAttribute(name, object)
 
     def check(self, object, name):
         'See IChecker'
@@ -150,16 +202,18 @@ class Checker(object):
                 return
             else:
                 __traceback_supplement__ = (TracebackSupplement, object)
-                raise Unauthorized(name, permission)
+                raise Unauthorized(object, name, permission)
         elif name in _available_by_default:
             return
 
         if name != '__iter__' or hasattr(object, name):
             __traceback_supplement__ = (TracebackSupplement, object)
-            raise ForbiddenAttribute, (name, object)
+            raise ForbiddenAttribute(name, object)
 
     def proxy(self, value):
         'See IChecker'
+        if type(value) is Proxy:
+            return value
         checker = getattr(value, '__Security_checker__', None)
         if checker is None:
             checker = selectChecker(value)
@@ -340,6 +394,22 @@ def defineChecker(type_, checker):
         raise DuplicationError(type_)
     _checkers[type_] = checker
 
+def defineChecker(type_, checker):
+    """Define a checker for a given type of object
+
+    The checker can be a Checker, or a function that, when called with
+    an object, returns a Checker.
+    """
+    if not isinstance(type_, (type, types.ClassType, types.ModuleType)):
+        raise TypeError(
+                'type_ must be a type, class or module, not a %s' % type_)
+    if type_ in _checkers:
+        raise DuplicationError(type_)
+    _checkers[type_] = checker
+
+def undefineChecker(type_):
+    del _checkers[type_]
+
 NoProxy = object()
 
 # _checkers is a mapping.
@@ -517,7 +587,35 @@ _namedChecker = NamesChecker(['__name__'])
 
 _iteratorChecker = NamesChecker(['next', '__iter__'])
 
-BasicTypes = {
+_setChecker = NamesChecker(['__iter__', '__len__', '__str__', '__contains__',
+                            'copy', 'difference', 'intersection', 'issubset',
+                            'issuperset', 'symmetric_difference', 'union',
+                            '__and__', '__or__', '__sub__', '__xor__',
+                            '__eq__', '__ne__', '__lt__', '__gt__',
+                            '__le__', '__ge__'])
+
+class BasicTypes(dict):
+    """Basic Types Dictionary
+
+    Make sure that the checkers a really updated, when a new type is added.
+    """
+    def __setitem__(self, name, value):
+        super(BasicTypes.__class__, self).__setitem__(name, value)
+        _checkers[name] = value
+
+    def __delitem__(self, name):
+        super(BasicTypes.__class__, self).__delitem__(name)
+        del _checkers[name]
+
+    def clear(self):
+        # Make sure you cannot clear the values
+        raise NotImplementedError
+
+    def update(self, d):
+        super(BasicTypes.__class__, self).update(d)
+        _checkers.update(d)
+
+BasicTypes = BasicTypes({
     object: NoProxy,
     int: NoProxy,
     float: NoProxy,
@@ -526,12 +624,14 @@ BasicTypes = {
     types.NoneType: NoProxy,
     str: NoProxy,
     unicode: NoProxy,
-    type(True): NoProxy, # Boolean, if available :)
+    bool: NoProxy,
     datetime.timedelta: NoProxy,
     datetime.datetime: NoProxy,
     datetime.date: NoProxy,
     datetime.time: NoProxy,
-}
+    datetime.tzinfo: NoProxy,
+    type(pytz.UTC): NoProxy,
+})
 
 # Available for tests. Located here so it can be kept in sync with BasicTypes.
 BasicTypes_examples = {
@@ -543,7 +643,7 @@ BasicTypes_examples = {
     types.NoneType: None,
     str: 'abc',
     unicode: u'uabc',
-    type(True): True,
+    bool: True,
     datetime.timedelta: datetime.timedelta(3),
     datetime.datetime: datetime.datetime(2003, 1, 1),
     datetime.date: datetime.date(2003, 1, 1),
@@ -572,6 +672,8 @@ _default_checkers = {
     list: NamesChecker(['__getitem__', '__getslice__', '__len__', '__iter__',
                         '__contains__', 'index', 'count', '__str__',
                         '__add__', '__radd__', ]),
+    sets.Set: _setChecker,
+    sets.ImmutableSet: _setChecker,
 
     # YAGNI: () a rock
     tuple: NamesChecker(['__getitem__', '__getslice__', '__add__', '__radd__',
@@ -599,7 +701,8 @@ _default_checkers = {
     type(Interface): InterfaceChecker(
         IInterface,
         __str__=CheckerPublic, _implied=CheckerPublic, subscribe=CheckerPublic,
-        # TODO: Backward:
+        # BBB 2004-07-13: backward compatability for deprecated
+        # interface methods
         isImplementedByInstancesOf=CheckerPublic,
         isImplementedBy=CheckerPublic,
         ),
@@ -611,6 +714,14 @@ _default_checkers = {
     zope.interface.declarations.Declaration: _Declaration_checker,
 }
 
+# If we are running with Python 2.4+, setup security on the builtin
+# set and frozenset types.
+try:
+    _default_checkers[set] = _setChecker
+    _default_checkers[frozenset] = _setChecker
+except NameError:
+    pass
+
 def _clear():
     _checkers.clear()
     _checkers.update(_default_checkers)
