@@ -13,39 +13,45 @@
 ##############################################################################
 """Interpreter for a pre-compiled TAL program.
 
-$Id: talinterpreter.py 30233 2005-05-03 08:09:03Z hdima $
+$Id: talinterpreter.py 39869 2005-11-03 14:23:04Z hdima $
 """
 import cgi
+import operator
 import sys
+import warnings
 
 # Do not use cStringIO here!  It's not unicode aware. :(
 from StringIO import StringIO
 
-from zope.i18nmessageid import MessageID
+# BBB 2005/10/10 -- MessageIDs are to be removed for Zope 3.3
+import zope.deprecation
+zope.deprecation.__show__.off()
+from zope.i18nmessageid import MessageID, Message
+zope.deprecation.__show__.on()
+
 from zope.tal.taldefs import quote, TAL_VERSION, METALError
 from zope.tal.taldefs import isCurrentVersion
 from zope.tal.taldefs import getProgramVersion, getProgramMode
 from zope.tal.talgenerator import TALGenerator
 from zope.tal.translationcontext import TranslationContext
 
-BOOLEAN_HTML_ATTRS = [
+
+# Avoid constructing this tuple over and over
+I18nMessageTypes = (MessageID, Message)
+
+TypesToTranslate = I18nMessageTypes + (str, unicode)
+
+# TODO: In Python 2.4 we can use frozenset() instead of dict.fromkeys()
+BOOLEAN_HTML_ATTRS = dict.fromkeys([
     # List of Boolean attributes in HTML that should be rendered in
     # minimized form (e.g. <img ismap> rather than <img ismap="">)
     # From http://www.w3.org/TR/xhtml1/#guidelines (C.10)
-    # XXX The problem with this is that this is not valid XML and
+    # TODO: The problem with this is that this is not valid XML and
     # can't be parsed back!
     "compact", "nowrap", "ismap", "declare", "noshade", "checked",
     "disabled", "readonly", "multiple", "selected", "noresize",
     "defer"
-]
-
-def _init():
-    d = {}
-    for s in BOOLEAN_HTML_ATTRS:
-        d[s] = 1
-    return d
-
-BOOLEAN_HTML_ATTRS = _init()
+])
 
 _nulljoin = ''.join
 _spacejoin = ' '.join
@@ -89,15 +95,102 @@ class AltTALGenerator(TALGenerator):
         return TALGenerator.replaceAttrs(self, attrlist, repldict)
 
 
+
+class MacroStackItem(list):
+    # This is a `list` subclass for backward compability.
+    """Stack entry for the TALInterpreter.macroStack.
+
+    This offers convenience attributes for more readable access.
+
+    """
+    __slots__ = ()
+
+    # These would be nicer using @syntax, but that would require
+    # Python 2.4.x; this will do for now.
+
+    macroName = property(lambda self: self[0])
+    slots = property(lambda self: self[1])
+    definingName = property(lambda self: self[2])
+    extending = property(lambda self: self[3])
+    entering = property(lambda self: self[4],
+                        lambda self, value: operator.setitem(self, 4, value))
+    i18nContext = property(lambda self: self[5])
+
+
 class TALInterpreter(object):
+    """TAL interpreter.
+
+    Some notes on source annotations.  They are HTML/XML comments added to the
+    output whenever sourceFile is changed by a setSourceFile bytecode.  Source
+    annotations are disabled by default, but you can turn them on by passing a
+    sourceAnnotations argument to the constructor.  You can change the format
+    of the annotations by overriding formatSourceAnnotation in a subclass.
+
+    The output of the annotation is delayed until some actual text is output
+    for two reasons:
+
+        1. setPosition bytecode follows setSourceFile, and we need position
+           information to output the line number.
+        2. Comments are not allowed in XML documents before the <?xml?>
+           declaration.
+
+    For performance reasons (TODO: premature optimization?) instead of checking
+    the value of _pending_source_annotation on every write to the output
+    stream, the _stream_write attribute is changed to point to
+    _annotated_stream_write method whenever _pending_source_annotation is
+    set to True, and to _stream.write when it is False.  The following
+    invariant always holds:
+
+        if self._pending_source_annotation:
+            assert self._stream_write is self._annotated_stream_write
+        else:
+            assert self._stream_write is self.stream.write
+
+    """
 
     def __init__(self, program, macros, engine, stream=None,
                  debug=0, wrap=60, metal=1, tal=1, showtal=-1,
-                 strictinsert=1, stackLimit=100, i18nInterpolate=1):
+                 strictinsert=1, stackLimit=100, i18nInterpolate=1,
+                 sourceAnnotations=0):
+        """Create a TAL interpreter.
+
+        Optional arguments:
+
+            stream -- output stream (defaults to sys.stdout).
+
+            debug -- enable debugging output to sys.stderr (off by default).
+
+            wrap -- try to wrap attributes on opening tags to this number of
+            column (default: 60).
+
+            metal -- enable METAL macro processing (on by default).
+
+            tal -- enable TAL processing (on by default).
+
+            showtal -- do not strip away TAL directives.  A special value of
+            -1 (which is the default setting) enables showtal when TAL
+            processing is disabled, and disables showtal when TAL processing is
+            enabled.  Note that you must use 0, 1, or -1; true boolean values
+            are not supported (TODO: why?).
+
+            strictinsert -- enable TAL processing and stricter HTML/XML
+            checking on text produced by structure inserts (on by default).
+            Note that Zope turns this value off by default.
+
+            stackLimit -- set macro nesting limit (default: 100).
+
+            i18nInterpolate -- enable i18n translations (default: on).
+
+            sourceAnnotations -- enable source annotations with HTML comments
+            (default: off).
+
+        """
         self.program = program
         self.macros = macros
         self.engine = engine # Execution engine (aka context)
         self.Default = engine.getDefault()
+        self._pending_source_annotation = False
+        self._currentTag = ""
         self._stream_stack = [stream or sys.stdout]
         self.popStream()
         self.debug = debug
@@ -117,7 +210,12 @@ class TALInterpreter(object):
         self.html = 0
         self.endsep = "/>"
         self.endlen = len(self.endsep)
+        # macroStack entries are MacroStackItem instances;
+        # the entries are mutated while on the stack
         self.macroStack = []
+        # `inUseDirective` is set iff we're handling either a
+        # metal:use-macro or a metal:extend-macro
+        self.inUseDirective = False
         self.position = None, None  # (lineno, offset)
         self.col = 0
         self.level = 0
@@ -126,6 +224,12 @@ class TALInterpreter(object):
         self.i18nStack = []
         self.i18nInterpolate = i18nInterpolate
         self.i18nContext = TranslationContext()
+        self.sourceAnnotations = sourceAnnotations
+
+    def StringIO(self):
+        # Third-party products wishing to provide a full Unicode-aware
+        # StringIO can do so by monkey-patching this method.
+        return FasterStringIO()
 
     def saveState(self):
         return (self.position, self.col, self.stream, self._stream_stack,
@@ -134,7 +238,10 @@ class TALInterpreter(object):
     def restoreState(self, state):
         (self.position, self.col, self.stream,
          self._stream_stack, scopeLevel, level, i18n) = state
-        self._stream_write = self.stream.write
+        if self._pending_source_annotation:
+            self._stream_write = self._annotated_stream_write
+        else:
+            self._stream_write = self.stream.write
         assert self.level == level
         while self.scopeLevel > scopeLevel:
             self.engine.endScope()
@@ -145,15 +252,20 @@ class TALInterpreter(object):
     def restoreOutputState(self, state):
         (dummy, self.col, self.stream,
          self._stream_stack, scopeLevel, level, i18n) = state
-        self._stream_write = self.stream.write
+        if self._pending_source_annotation:
+            self._stream_write = self._annotated_stream_write
+        else:
+            self._stream_write = self.stream.write
         assert self.level == level
         assert self.scopeLevel == scopeLevel
 
-    def pushMacro(self, macroName, slots, entering=1):
+    def pushMacro(self, macroName, slots, definingName, extending):
         if len(self.macroStack) >= self.stackLimit:
             raise METALError("macro nesting limit (%d) exceeded "
                              "by %s" % (self.stackLimit, `macroName`))
-        self.macroStack.append([macroName, slots, entering, self.i18nContext])
+        self.macroStack.append(
+            MacroStackItem((macroName, slots, definingName, extending,
+                            True, self.i18nContext)))
 
     def popMacro(self):
         return self.macroStack.pop()
@@ -173,11 +285,43 @@ class TALInterpreter(object):
     def pushStream(self, newstream):
         self._stream_stack.append(self.stream)
         self.stream = newstream
-        self._stream_write = newstream.write
+        if self._pending_source_annotation:
+            self._stream_write = self._annotated_stream_write
+        else:
+            self._stream_write = self.stream.write
 
     def popStream(self):
         self.stream = self._stream_stack.pop()
+        if self._pending_source_annotation:
+            self._stream_write = self._annotated_stream_write
+        else:
+            self._stream_write = self.stream.write
+
+    def _annotated_stream_write(self, s):
+        idx = s.find('<?xml')
+        if idx >= 0 or s.isspace():
+            # Do not preprend comments in front of the <?xml?> declaration.
+            end_of_doctype = s.find('?>', idx)
+            if end_of_doctype > idx:
+                self.stream.write(s[:end_of_doctype+2])
+                s = s[end_of_doctype+2:]
+                # continue
+            else:
+                self.stream.write(s)
+                return
+        self._pending_source_annotation = False
         self._stream_write = self.stream.write
+        self._stream_write(self.formatSourceAnnotation())
+        self._stream_write(s)
+
+    def formatSourceAnnotation(self):
+        lineno = self.position[0]
+        if lineno is None:
+            location = self.sourceFile
+        else:
+            location = '%s (line %s)' % (self.sourceFile, lineno)
+        sep = '=' * 78
+        return '<!--\n%s\n%s\n%s\n-->' % (sep, location, sep)
 
     def stream_write(self, s,
                      len=len):
@@ -226,6 +370,10 @@ class TALInterpreter(object):
     def do_setSourceFile(self, source_file):
         self.sourceFile = source_file
         self.engine.setSourceFile(source_file)
+        if self.sourceAnnotations:
+            self._pending_source_annotation = True
+            self._stream_write = self._annotated_stream_write
+
     bytecode_handlers["setSourceFile"] = do_setSourceFile
 
     def do_setPosition(self, position):
@@ -255,28 +403,29 @@ class TALInterpreter(object):
         try:
             for item in attrList:
                 if _len(item) == 2:
-                    name, s = item
+                    rendered = item[1:]
                 else:
                     # item[2] is the 'action' field:
                     if item[2] in ('metal', 'tal', 'xmlns', 'i18n'):
                         if not self.showtal:
                             continue
-                        ok, name, s = self.attrAction(item)
+                        rendered = self.attrAction(item)
                     else:
-                        ok, name, s = attrAction(self, item)
-                    if not ok:
+                        rendered = attrAction(self, item)
+                    if not rendered:
                         continue
-                slen = _len(s)
-                if (wrap and
-                    col >= align and
-                    col + 1 + slen > wrap):
-                    append("\n")
-                    append(" "*align)
-                    col = align + slen
-                else:
-                    append(" ")
-                    col = col + 1 + slen
-                append(s)
+                for s in rendered:
+                    slen = _len(s)
+                    if (wrap and
+                        col >= align and
+                        col + 1 + slen > wrap):
+                        append("\n")
+                        append(" "*align)
+                        col = align + slen
+                    else:
+                        append(" ")
+                        col = col + 1 + slen
+                    append(s)
             append(end)
             col = col + endlen
         finally:
@@ -287,33 +436,43 @@ class TALInterpreter(object):
     def attrAction(self, item):
         name, value, action = item[:3]
         if action == 'insert':
-            return 0, name, value
+            return ()
         macs = self.macroStack
         if action == 'metal' and self.metal and macs:
-            if len(macs) > 1 or not macs[-1][2]:
-                # Drop all METAL attributes at a use-depth above one.
-                return 0, name, value
-            # Clear 'entering' flag
-            macs[-1][2] = 0
+            # Drop all METAL attributes at a use-depth beyond the first
+            # use-macro and its extensions
+            if len(macs) > 1:
+                for macro in macs[1:]:
+                    if not macro.extending:
+                        return ()
+            if not macs[-1].entering:
+                return ()
+            macs[-1].entering = False
             # Convert or drop depth-one METAL attributes.
             i = name.rfind(":") + 1
             prefix, suffix = name[:i], name[i:]
             if suffix == "define-macro":
                 # Convert define-macro as we enter depth one.
-                name = prefix + "use-macro"
-                value = macs[-1][0] # Macro name
+                useName = macs[0].macroName
+                defName = macs[0].definingName
+                res = []
+                if defName:
+                    res.append('%sdefine-macro=%s' % (prefix, quote(defName)))
+                if useName:
+                    res.append('%suse-macro=%s' % (prefix, quote(useName)))
+                return res
             elif suffix == "define-slot":
                 name = prefix + "fill-slot"
             elif suffix == "fill-slot":
                 pass
             else:
-                return 0, name, value
+                return ()
 
         if value is None:
             value = name
         else:
             value = "%s=%s" % (name, quote(value))
-        return 1, name, value
+        return [value]
 
     def attrAction_tal(self, item):
         name, value, action = item[:3]
@@ -340,18 +499,19 @@ class TALInterpreter(object):
 
         if ok:
             if xlat:
-                translated = self.translate(msgid or value, value, {})
+                translated = self.translate(msgid or value, value)
                 if translated is not None:
                     value = translated
             if value is None:
                 value = name
-            value = "%s=%s" % (name, quote(value))
-        return ok, name, value
+            return ["%s=%s" % (name, quote(value))]
+        else:
+            return ()
     bytecode_handlers["<attrAction>"] = attrAction
 
     def no_tag(self, start, program):
         state = self.saveState()
-        self.stream = stream = StringIO()
+        self.stream = stream = self.StringIO()
         self._stream_write = stream.write
         self.interpret(start)
         self.restoreOutputState(state)
@@ -447,6 +607,19 @@ class TALInterpreter(object):
 
     def do_insertText(self, stuff):
         self.interpret(stuff[1])
+    bytecode_handlers["insertText"] = do_insertText
+    bytecode_handlers["insertI18nText"] = do_insertText
+
+    def _writeText(self, text):
+        # '&' must be done first!
+        s = text.replace(
+            "&", "&amp;").replace("<", "&lt;").replace(">", "&gt;")
+        self._stream_write(s)
+        i = s.rfind('\n')
+        if i < 0:
+            self.col += len(s)
+        else:
+            self.col = len(s) - (i + 1)
 
     def do_insertText_tal(self, stuff):
         text = self.engine.evaluateText(stuff[0])
@@ -455,19 +628,36 @@ class TALInterpreter(object):
         if text is self.Default:
             self.interpret(stuff[1])
             return
-        if isinstance(text, MessageID):
+        if isinstance(text, I18nMessageTypes):
             # Translate this now.
-            text = self.engine.translate(text)
-        # '&' must be done first!
-        s = text.replace(
-            "&", "&amp;").replace("<", "&lt;").replace(">", "&gt;")
-        self._stream_write(s)
-        i = s.rfind('\n')
-        if i < 0:
-            self.col = self.col + len(s)
+            # BBB: Deprecated. Will be removed in 3.3
+            self._i18n_deprecate()
+            text = self.translate(text)
+        self._writeText(text)
+
+    def _i18n_deprecate(self):
+        if self.sourceFile is None:
+            source = "<unknown>"
         else:
-            self.col = len(s) - (i + 1)
-    bytecode_handlers["insertText"] = do_insertText
+            source = self.sourceFile
+        lineno = self.position[0]
+        if lineno is not None:
+            source += " (line %s)" % lineno
+        warnings.warn('%s: Automatic translation of message id\'s is'
+            ' deprecated and will be removed in 3.3.'
+            ' Use explicit i18n:translate="" instead.'
+            % source, DeprecationWarning, 2)
+
+    def do_insertI18nText_tal(self, stuff):
+        # TODO: Code duplication is BAD, we need to fix it later
+        text = self.engine.evaluateText(stuff[0])
+        if text is not None:
+            if text is self.Default:
+                self.interpret(stuff[1])
+            else:
+                if isinstance(text, TypesToTranslate):
+                    text = self.translate(text)
+                self._writeText(text)
 
     def do_i18nVariable(self, stuff):
         varname, program, expression, structure = stuff
@@ -476,7 +666,7 @@ class TALInterpreter(object):
             # evaluate the mini-program to get the value of the variable.
             state = self.saveState()
             try:
-                tmpstream = StringIO()
+                tmpstream = self.StringIO()
                 self.pushStream(tmpstream)
                 try:
                     self.interpret(program)
@@ -489,6 +679,9 @@ class TALInterpreter(object):
             finally:
                 self.restoreState(state)
         else:
+            # TODO: Seems like this branch not used anymore, we
+            # need to remove it
+
             # Evaluate the value to be associated with the variable in the
             # i18n interpolation dictionary.
             if structure:
@@ -496,10 +689,12 @@ class TALInterpreter(object):
             else:
                 value = self.engine.evaluate(expression)
 
-            # evaluate() does not do any I18n, so we do it here. 
-            if isinstance(value, MessageID):
+            # evaluate() does not do any I18n, so we do it here.
+            if isinstance(value, I18nMessageTypes):
                 # Translate this now.
-                value = self.engine.translate(value)
+                # BBB: Deprecated. Will be removed in 3.3
+                self._i18n_deprecate()
+                value = self.translate(value)
 
             if not structure:
                 value = cgi.escape(unicode(value))
@@ -531,7 +726,7 @@ class TALInterpreter(object):
         # Use a temporary stream to capture the interpretation of the
         # subnodes, which should /not/ go to the output stream.
         currentTag = self._currentTag
-        tmpstream = StringIO()
+        tmpstream = self.StringIO()
         self.pushStream(tmpstream)
         try:
             self.interpret(stuff[1])
@@ -551,8 +746,8 @@ class TALInterpreter(object):
         if len(stuff) > 2:
             obj = self.engine.evaluate(stuff[2])
         xlated_msgid = self.translate(msgid, default, i18ndict, obj)
-        # XXX I can't decide whether we want to cgi escape the translated
-        # string or not.  OT1H not doing this could introduce a cross-site
+        # TODO: I can't decide whether we want to cgi escape the translated
+        # string or not.  OTOH not doing this could introduce a cross-site
         # scripting vector by allowing translators to sneak JavaScript into
         # translations.  OTOH, for implicit interpolation values, we don't
         # want to escape stuff like ${name} <= "<b>Timmy</b>".
@@ -562,6 +757,8 @@ class TALInterpreter(object):
 
     def do_insertStructure(self, stuff):
         self.interpret(stuff[2])
+    bytecode_handlers["insertStructure"] = do_insertStructure
+    bytecode_handlers["insertI18nStructure"] = do_insertStructure
 
     def do_insertStructure_tal(self, (expr, repldict, block)):
         structure = self.engine.evaluateStructure(expr)
@@ -579,7 +776,25 @@ class TALInterpreter(object):
             self.insertHTMLStructure(text, repldict)
         else:
             self.insertXMLStructure(text, repldict)
-    bytecode_handlers["insertStructure"] = do_insertStructure
+
+    def do_insertI18nStructure_tal(self, (expr, repldict, block)):
+        # TODO: Code duplication is BAD, we need to fix it later
+        structure = self.engine.evaluateStructure(expr)
+        if structure is not None:
+            if structure is self.Default:
+                self.interpret(block)
+            else:
+                if isinstance(structure, TypesToTranslate):
+                    text = self.translate(structure)
+                else:
+                    text = unicode(structure)
+                if not (repldict or self.strictinsert):
+                    # Take a shortcut, no error checking
+                    self.stream_write(text)
+                elif self.html:
+                    self.insertHTMLStructure(text, repldict)
+                else:
+                    self.insertXMLStructure(text, repldict)
 
     def insertHTMLStructure(self, text, repldict):
         from zope.tal.htmltalparser import HTMLTALParser
@@ -606,7 +821,7 @@ class TALInterpreter(object):
         lang, program = stuff
         # Use a temporary stream to capture the interpretation of the
         # subnodes, which should /not/ go to the output stream.
-        tmpstream = StringIO()
+        tmpstream = self.StringIO()
         self.pushStream(tmpstream)
         try:
             self.interpret(program)
@@ -614,7 +829,7 @@ class TALInterpreter(object):
             self.popStream()
         code = tmpstream.getvalue()
         output = self.engine.evaluateCode(lang, code)
-        self._stream_write(output)        
+        self._stream_write(output)
     bytecode_handlers["evaluateCode"] = do_evaluateCode
 
     def do_loop(self, (name, expr, block)):
@@ -626,12 +841,19 @@ class TALInterpreter(object):
             self.interpret(block)
     bytecode_handlers["loop"] = do_loop
 
-    def translate(self, msgid, default, i18ndict, obj=None):
+    def translate(self, msgid, default=None, i18ndict=None,
+                  obj=None, domain=None):
+        if default is None:
+            default = getattr(msgid, 'default', unicode(msgid))
+        if i18ndict is None:
+            i18ndict = {}
+        if domain is None:
+            domain = getattr(msgid, 'domain', self.i18nContext.domain)
         if obj:
             i18ndict.update(obj)
         if not self.i18nInterpolate:
             return msgid
-        # XXX We need to pass in one of context or target_language
+        # TODO: We need to pass in one of context or target_language
         return self.engine.translate(msgid, self.i18nContext.domain,
                                      i18ndict, default=default)
 
@@ -651,19 +873,14 @@ class TALInterpreter(object):
     bytecode_handlers["condition"] = do_condition
 
     def do_defineMacro(self, (macroName, macro)):
-        macs = self.macroStack
-        if len(macs) == 1:
-            entering = macs[-1][2]
-            if not entering:
-                macs.append(None)
-                self.interpret(macro)
-                assert macs[-1] is None
-                macs.pop()
-                return
+        wasInUse = self.inUseDirective
+        self.inUseDirective = False
         self.interpret(macro)
+        self.inUseDirective = wasInUse
     bytecode_handlers["defineMacro"] = do_defineMacro
 
-    def do_useMacro(self, (macroName, macroExpr, compiledSlots, block)):
+    def do_useMacro(self, (macroName, macroExpr, compiledSlots, block),
+                    definingName=None, extending=False):
         if not self.metal:
             self.interpret(block)
             return
@@ -679,14 +896,18 @@ class TALInterpreter(object):
             if mode != (self.html and "html" or "xml"):
                 raise METALError("macro %s has incompatible mode %s" %
                                  (`macroName`, `mode`), self.position)
-        self.pushMacro(macroName, compiledSlots)
-        
-        # We want 'macroname' name to be always available as a variable 
+        self.pushMacro(macroName, compiledSlots, definingName, extending)
+
+        # We want 'macroname' name to be always available as a variable
         outer = self.engine.getValue('macroname')
         self.engine.setLocal('macroname', macroName.split('/')[-1])
 
         prev_source = self.sourceFile
+        wasInUse = self.inUseDirective
+        self.inUseDirective = True
         self.interpret(macro)
+        self.inUseDirective = wasInUse
+
         if self.sourceFile != prev_source:
             self.engine.setSourceFile(prev_source)
             self.sourceFile = prev_source
@@ -695,6 +916,16 @@ class TALInterpreter(object):
         self.engine.setLocal('macroname', outer)
     bytecode_handlers["useMacro"] = do_useMacro
 
+    def do_extendMacro(self, (macroName, macroExpr, compiledSlots, block,
+                              definingName)):
+        # extendMacro results from a combination of define-macro and
+        # use-macro.  definingName has the value of the
+        # metal:define-macro attribute.
+        extending = self.metal and self.inUseDirective
+        self.do_useMacro((macroName, macroExpr, compiledSlots, block),
+                         definingName, extending)
+    bytecode_handlers["extendMacro"] = do_extendMacro
+
     def do_fillSlot(self, (slotName, block)):
         # This is only executed if the enclosing 'use-macro' evaluates
         # to 'default'.
@@ -706,18 +937,41 @@ class TALInterpreter(object):
             self.interpret(block)
             return
         macs = self.macroStack
-        if macs and macs[-1] is not None:
-            macroName, slots = self.popMacro()[:2]
-            slot = slots.get(slotName)
+        if macs:
+            len_macs = len(macs)
+            # Measure the extension depth of this use-macro
+            depth = 1
+            while depth < len_macs:
+                if macs[-depth].extending:
+                    depth += 1
+                else:
+                    break
+            # Search for a slot filler from the most specific to the
+            # most general macro.  The most general is at the top of
+            # the stack.
+            slot = None
+            i = len_macs - 1
+            while i >= (len_macs - depth):
+                slot = macs[i].slots.get(slotName)
+                if slot is not None:
+                    break
+                i -= 1
             if slot is not None:
+                # Found a slot filler.  Temporarily chop the macro
+                # stack starting at the macro that filled the slot and
+                # render the slot filler.
+                chopped = macs[i:]
+                del macs[i:]
                 prev_source = self.sourceFile
                 self.interpret(slot)
                 if self.sourceFile != prev_source:
                     self.engine.setSourceFile(prev_source)
                     self.sourceFile = prev_source
-                self.pushMacro(macroName, slots, entering=0)
+                # Restore the stack entries.
+                for mac in chopped:
+                    mac.entering = False  # Not entering
+                macs.extend(chopped)
                 return
-            self.pushMacro(macroName, slots)
             # Falling out of the 'if' allows the macro to be interpreted.
         self.interpret(block)
     bytecode_handlers["defineSlot"] = do_defineSlot
@@ -727,10 +981,11 @@ class TALInterpreter(object):
 
     def do_onError_tal(self, (block, handler)):
         state = self.saveState()
-        self.stream = stream = StringIO()
+        self.stream = stream = self.StringIO()
         self._stream_write = stream.write
         try:
             self.interpret(block)
+        # TODO: this should not catch ZODB.POSException.ConflictError.
         except:
             exc = sys.exc_info()[1]
             self.restoreState(state)
@@ -753,8 +1008,33 @@ class TALInterpreter(object):
     bytecode_handlers_tal["setLocal"] = do_setLocal_tal
     bytecode_handlers_tal["setGlobal"] = do_setGlobal_tal
     bytecode_handlers_tal["insertStructure"] = do_insertStructure_tal
+    bytecode_handlers_tal["insertI18nStructure"] = do_insertI18nStructure_tal
     bytecode_handlers_tal["insertText"] = do_insertText_tal
+    bytecode_handlers_tal["insertI18nText"] = do_insertI18nText_tal
     bytecode_handlers_tal["loop"] = do_loop_tal
     bytecode_handlers_tal["onError"] = do_onError_tal
     bytecode_handlers_tal["<attrAction>"] = attrAction_tal
     bytecode_handlers_tal["optTag"] = do_optTag_tal
+
+
+class FasterStringIO(StringIO):
+    """Append-only version of StringIO.
+
+    This let's us have a much faster write() method.
+    """
+    def close(self):
+        if not self.closed:
+            self.write = _write_ValueError
+            StringIO.close(self)
+
+    def seek(self, pos, mode=0):
+        raise RuntimeError("FasterStringIO.seek() not allowed")
+
+    def write(self, s):
+        #assert self.pos == self.len
+        self.buflist.append(s)
+        self.len = self.pos = self.pos + len(s)
+
+
+def _write_ValueError(s):
+    raise ValueError("I/O operation on closed file")
